/*******************************************************************************
* FILE NAME: OI_routines.c
*
* DESCRIPTION:
*    Update the OI display LEDs and the dashboard status information
*
*******************************************************************************/

#ifdef _DEBUG
#include "print_routines.h"
#endif

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"

#include "sensor_routines.h"
#include "user_routines.h"
#include "motor_routines.h"

#include "clock_routines.h"

#define BACKUPLOWVOLTWARNING 60  // Voltages are in tenths of volts
#define LOWVOLTAGEWARNING    110  // Voltages are in tenths of volts

void Update_OI()
{
  int  igyrovalue;
  static long Last_Encoder=0;
  long ltemp;
  static unsigned long LastClock;
  long distance, velocity;
  static unsigned char peakVelocity = 0;
  static long LastDistance=0;


 /*---------- ROBOT FEEDBACK LEDs------------------------------------------------
  *------------------------------------------------------------------------------
  *   This section drives the "ROBOT FEEDBACK" lights on the Operator Interface.
  *   The lights are green for joystick forward and red for joystick reverse.
  *   Both red and green are on when the joystick is centered.  Use the
  *   trim tabs on the joystick to adjust the center.     
  */	
  
  if (user_display_mode == OFF) /* User Mode is Off */
  { 
/**** Check trim position of Port 1 Arm Joystick ****/
    if (ARM_INPUT > -NULLJOYSTICK) 
      Pwm1_green  = ON;    /* Turn PWM1 green LED - OFF */
    else
      Pwm1_green  = OFF;    /* Turn PWM1 green LED - OFF */
    if (ARM_INPUT <  NULLJOYSTICK) 
      Pwm1_red  = ON;      /* Turn PWM1 red LED   - ON */
    else
      Pwm1_red  = OFF;      /* Turn PWM1 red LED   - ON */

    if (!OISWITCH1) // One joystick mode
    {
/**** Check trim position of Port 4 Drive x Joystick ****/
      if (JOY_DRIVE1B > -NULLJOYSTICK) 
        Pwm2_green  = ON;    /* Turn green LED - OFF */
      else
        Pwm2_green  = OFF;    /* Turn green LED - OFF */

      if (JOY_DRIVE1B <  NULLJOYSTICK) 
       Pwm2_red  = ON;      /* Turn red LED   - ON */
      else
        Pwm2_red  = OFF;      /* Turn red LED   - ON */
    }
    else  // Two joystick mode
    {
/**** Check trim position of Port 3 Drive y Joystick ****/
      if (JOY_DRIVE2 > -NULLJOYSTICK) 
        Pwm2_green  = ON;    /* Turn green LED - OFF */
      else
        Pwm2_green  = OFF;    /* Turn green LED - OFF */

      if (JOY_DRIVE2 <  NULLJOYSTICK) 
       Pwm2_red  = ON;      /* Turn red LED   - ON */
      else
        Pwm2_red  = OFF;      /* Turn red LED   - ON */
    }

/**** Check trim position of Port 4 Drive y Joystick ****/
    if (JOY_DRIVE1A > -NULLJOYSTICK) 
      Relay1_green  = ON;    /* Turn Relay1 green LED - OFF */
    else
      Relay1_green  = OFF;    /* Turn Relay1 green LED - OFF */

    if (JOY_DRIVE1A <  NULLJOYSTICK) 
      Relay1_red  = ON;      /* Turn Relay1 red LED   - ON */
    else
      Relay1_red  = OFF;      /* Turn Relay1 red LED   - ON */

/**** Warning lights for low battery power ****/
     if (backup_voltage < BACKUPLOWVOLTWARNING)  // Voltages are *10
       Relay2_green = ON;  // Warning when backup battery voltage is low */
     else
       Relay2_green = OFF;
     if (battery_voltage < LOWVOLTAGEWARNING)  // Voltages are *100
       Relay2_red = ON;  // Warning when main battery voltage is low */
     else
       Relay2_red = OFF;
     if (LastClock < Clocksec)
     {
#ifdef _DEBUG
//       PrintString((char *) "backup_voltage="); PrintDecimal(backup_voltage);
//       PrintString((char *) "main_voltage="); PrintDecimal(battery_voltage);
//       PrintCR();
#endif
     }
  } /* (user_display_mode = 0) (User Mode is Off) */
  
  else  /* User Mode is On - displays data in OI 4-digit display*/
  {

 /*---------- ROBOT FEEDBACK User Mode Display-----------------------------------
  *------------------------------------------------------------------------------
  */


//    User_Mode_byte = backup_voltage*10; /* so that decimal doesn't get truncated. */

//  The FIELD position switch is used within certain functions to reverse script 
//  cmd parameters mirroring operations on the opposite side of the field
//  Commands affected include: DSPIN, DISTTURN, GTURN, GTURNGOAL
    User_Mode_byte = PORTJ>>4;  // Autonomous selected
    if (rc_dig_in11 == 0) 
    {
      User_Mode_byte += 100; // Left
    }
    else if (rc_dig_in12 == 0)
    {
      User_Mode_byte += 200; // Center
    }
    else if (rc_dig_in13 == 0)
    {
//      User_Mode_byte += 0; // Right
    }
  }

  /**** Miscellaneous indicators ****/
//  Switch1_LED = rc_dig_in09; // Red/Blue switch - on = red
  Switch1_LED = OFF; // Red/Blue switch - on = red
  Switch2_LED = PUSH_DOWN_PADS; // Goal hook is down
  Switch3_LED = relay8_fwd; // Compressor on

 /*---------- Dataport Display --------------------------------------------------
  *------------------------------------------------------------------------------
  */

  User_Byte1  = OI_KP_POT>>1; // OI pot for Kp 
  User_Byte2  = OI_KI_POT>>1; // OI pot for Ki
  User_Byte5  = OI_KD_POT>>1; // OI pot for Kd

  /* Display the gyro heading byte3 if its negative, byte4 if its positive */
  igyrovalue = GYRO_DEGREES/10; // convert from tenths of degrees to whole degrees
  if (igyrovalue > 0)
  {
  User_Byte3  = 0; // Gyro degrees
  User_Byte4  = (char) igyrovalue;
  }
  else
  {
  User_Byte3  = (char) -igyrovalue; // Gyro degrees
  User_Byte4  = 0;
  }

  /* min/max the gyro has hit - to check for maxing out */
//  temp = -min_gyro/2 - 1;
//  User_Byte5  = (char) temp; // Turn rate minimum is -260
//  temp = max_gyro/2 - 1;
//  User_Byte6  = (char) temp; // Turn rate maximum is ~251


  if (LastClock < Clocksec)
  {
    LastClock = Clocksec;
    // (2)encoders * 16.28"(one encoder rev) / 2(encoders) / 1000(3 decimal places) / 12(inches per foot)
    distance = (Right_Encoder_Count+ Left_Encoder_Count)/2 * ENCODER_INCHES / TICKS_PER_REV; // inches
    if (distance < 0) distance *= -1; // Lets keep it positive

    /* velocity averaged over 1 sec. */
    velocity = distance - LastDistance; // inches per second
    LastDistance = distance;
    if (velocity > 255) velocity = 255;
    if (peakVelocity < (unsigned char) velocity)
    {
      peakVelocity = (unsigned char)velocity;
    }
    User_Byte6  = peakVelocity; // Velocity in ips
  }  
}
