/*******************************************************************************
* FILE NAME: auto_drive_routines.c
*
*	Version 4.0	last updated 8/2/04::
*
* DESCRIPTION:
*  this is the autonomous mode place
*   this is where the autonomous mode is places  
*
* USAGE:
*  Just use it...
*
*******************************************************************************/

#ifdef _DEBUG
#include "print_routines.h"
#endif

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"

#include "motor_routines.h"
#include "auto_routines.h"
#include "user_routines.h"
#include "clock_routines.h"
#include "sensor_routines.h"


/*******************************************************************************
* FUNCTION NAME: Gyro_Turn
* PURPOSE:       Turn the robot to the specified heading.
* ARGUMENTS:     input: speed 0-127
*
*                A 90 degree turn will center the back edge where the old back edge was 
*                and will lineup with the old left side. This may vary depending on tire traction.
*                 
*******************************************************************************/
#define GYROWINDOW 15 // tenths of degrees - Considered complete at gyro +/- this window of accuracy (waqs 4)
#define RESTFINISH 5 // In units of 1/40 sec., how long we must be on the setpoint
                     // (robot at rest) to be considered finished with this move
void Gyro_Turn(int heading)
{
  int gyrodegrees, temp, pid;
  static char finishCount;
  static int our_heading;
  char Success;

  if (lastDriveCmd != GTURN)
  {
    lastDriveCmd = GTURN;
//    if (RedBlue == BLUE)
//      heading = -heading;

    our_heading = heading * 10; // Convert to tenths of degrees
#ifdef _DEBUG
    PrintString((char *) " **Gyro_Turn** ");
    PrintCR();
#endif

    /** Assume heading is based on left field action **/
    /** Reverse if robot is on the right side of the field **/
    Gyro_PID(our_heading, TRUE);
    finishCount = 0;
  }

/* Put the turn pads down while turning */
    PUSH_DOWN_PADS = 1;
    PUSH_DOWN_PADS_REV = 0;

  gyrodegrees = GYRO_DEGREES;
  /** Be aware that if the gyro is at 180 you
   *  can order a turn to 270 or -90 and the robot will get to the same place
   *  just by spinning in opposite directions */

  Success = ((our_heading <= (gyrodegrees + GYROWINDOW)) & (our_heading >= (gyrodegrees - GYROWINDOW)));

  PrintString((char *) " our_heading=");
  PrintSignedDecimal(our_heading);
  PrintString((char *) ", gyrodegrees=");
  PrintSignedDecimal(gyrodegrees);
  PrintCR();

  if (Success & (finishCount == RESTFINISH))
  {
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;
    /* Return to earth */
    PadsCounter = 0; // To suspend Hold_Position while we are airborne
    PUSH_DOWN_PADS = 0;
    PUSH_DOWN_PADS_REV = 0;
    lastDriveCmd = ENDCMD;  // All done
    Drive_Status = COMPLETE;
#ifdef _DEBUG
    PrintString((char *) " gyro is done ");
    PrintCR();
#endif
  }
  else
  {
    if (Success) // Don't want to quit if we're just flying by the setpoint
      finishCount ++;
    else
      finishCount = 0;

    Drive_Status = INCOMPLETE;
    pid = Gyro_PID(our_heading, FALSE);
    WHL_L1 = pid;
  
    if (WHL_L1 < -HALFSPEED) WHL_L1 = -HALFSPEED;
    if (WHL_L1 > HALFSPEED) WHL_L1 = HALFSPEED;
//    if (WHL_L1 < -FULLSPEED) WHL_L1 = -FULLSPEED;
//    if (WHL_L1 > FULLSPEED) WHL_L1 = FULLSPEED;
    WHL_R1 = -WHL_L1;
  }
#ifdef _DEBUG
  PrintString((char *) ", gyro = ");  PrintSignedDecimal(gyrodegrees);
  PrintString((char *) ", our_heading = ");  PrintSignedDecimal(our_heading);
  PrintString((char *) ", WHL_L1 = ");  PrintSignedDecimal(WHL_L1);
  PrintString((char *) ", Clockds = ");  PrintSignedDecimal(Clockds);
  PrintCR();
#endif
}



/*******************************************************************************
* FUNCTION NAME: Drive_Forward_Time
* PURPOSE:       Drive straight for a given time at a given speed.
*                Pure time 1 sec = 87", 2 sec = 199.5" = top speed 9.38 fps
*                Each function determines completion or failure.
* SENSORS:       Gyro - for keeping straight
*******************************************************************************/
void Drive_Forward_Time(int time, int speed)
{
  static int desired_heading;
  int actual;
  static unsigned long duration; //in tenths of seconds
  static int LastActual, CumError;
  int PID;
 
  if (lastDriveCmd != DFWD)
  {
    lastDriveCmd = DFWD;
    duration = Clockds + time; // Current time + duration = cmd completed

    /* Keep straight using the gyro */
    desired_heading = GYRO_DEGREES; // This will be the heading to maintain
    CumError = 0;
    LastActual = 0;

    // Debug only
    INTCONbits.GIEL = 0;
    Right_Encoder_Count = 0;
    Left_Encoder_Count = 0;
    INTCONbits.GIEL = 1;
#ifdef _DEBUG
    PrintString((char *) " **Drive_Forward_Time** ");
    PrintCR();
#endif
  }

  if (duration > Clockds)
  { // Keep going
    Drive_Status = INCOMPLETE;

    actual = GYRO_DEGREES;
    PID = Gyro_Steering(desired_heading, actual, &LastActual, &CumError);

    /* Balance the motors */
    WHL_L1 = -speed;
    WHL_R1 = WHL_L1; 

    WHL_L1 += PWMBalance(BalForLeft, speed) - PID; //Maintain right speed. Make the left keep in sync.
    WHL_R1 += PWMBalance(BalForRight, speed);
    if (WHL_R1 < -127) WHL_R1 = -127;
    if (WHL_L1 < -127) WHL_L1 = -127;

#ifdef _DEBUG
    PrintString((char *) ", Clockms="); PrintSignedDecimal(Clockms);
    PrintString((char *) ", Re="); PrintSignedDecimal(Right_Encoder_Count);
    PrintString((char *) ", Le="); PrintSignedDecimal(Left_Encoder_Count);
    PrintString((char *) ", R1="); PrintSignedDecimal(WHL_R1);
    PrintString((char *) ", L1="); PrintSignedDecimal(WHL_L1);
    PrintCR();
#endif
  }
  else
  { // All done (move on to next drive command)
    Drive_Status = COMPLETE;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;
  }
}

/*******************************************************************************
* FUNCTION NAME: Drive_Backward_Time
* PURPOSE:       Function-based autonomous mode.
* CALLED FROM:   file:user_routines_fast.c, here
* ARGUMENTS:     input: speed 0-127
*                output: 
* RETURNS:       void
*******************************************************************************/
void Drive_Backward_Time(int time, int speed)
{
  static int desired_heading;
  int actual;
  static unsigned long duration; //in tenths of seconds
  static int LastActual, CumError;
  int PID;


  if (lastDriveCmd != DBCK)
  {
    lastDriveCmd = DBCK;
    duration = Clockds + time; // Current time + duration = cmd completed

    /* Keep straight using the gyro */
    desired_heading = GYRO_DEGREES; // This will be the heading to maintain
    CumError = 0;
    LastActual = 0;

    // Debug only
    INTCONbits.GIEL = 0;
    Right_Encoder_Count = 0;
    Left_Encoder_Count = 0;
    INTCONbits.GIEL = 1;
#ifdef _DEBUG
    PrintString((char *) " **Drive_Backward_Time** ");
    PrintCR();
#endif
  }

  if (duration > Clockds)
  { // Keep going
    Drive_Status = INCOMPLETE;
    actual = GYRO_DEGREES;
    PID = Gyro_Steering(desired_heading, actual, &LastActual, &CumError);

    WHL_L1 = speed;
    WHL_R1 = WHL_L1;
    WHL_L1 -= PWMBalance(BalBckLeft, speed);
    WHL_R1 -= (PWMBalance(BalBckRight, speed) - PID);
    if (WHL_R1 > 127) WHL_R1 = 127;
    if (WHL_L1 > 127) WHL_L1 = 127;
    if (WHL_R1 < 0) WHL_R1 = 0;
    if (WHL_L1 < 0) WHL_L1 = 0;

#ifdef _DEBUG
    PrintString((char *) ", Clockms="); PrintSignedDecimal(Clockms);
    PrintString((char *) ", Re="); PrintSignedDecimal(Right_Encoder_Count);
    PrintString((char *) ", Le="); PrintSignedDecimal(Left_Encoder_Count);
    PrintString((char *) ", R1="); PrintSignedDecimal(WHL_R1);
    PrintString((char *) ", L1="); PrintSignedDecimal(WHL_L1);
    PrintCR();
#endif
  }
  else
  { // All done (move on to next drive command)
    Drive_Status = COMPLETE;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;
  }
}



/*******************************************************************************
* FUNCTION NAME: Drive_Spin_In_Place_Time
* PURPOSE:       Function-based autonomous mode.
* CALLED FROM:   file:user_routines_fast.c, here
*******************************************************************************/
void Drive_Spin_In_Place_Time(int time, int speed, int direction)
{
  static unsigned long duration; //in tenths of seconds

 
  if (lastDriveCmd != DSPIN)
  {
    lastDriveCmd = DSPIN;
    Drive_Status = INCOMPLETE;
    duration = Clockds + time; // Current time + duration = cmd completed

#ifdef _DEBUG
    PrintString((char *) " **Drive_Spin_In_Place_Time** ");
    PrintCR();
#endif
  }
/* Put the turn pads down while turning */
    PUSH_DOWN_PADS = 1;
    PUSH_DOWN_PADS_REV = 0;

  if (duration > Clockds)
  { // Keep going
    Drive_Status = INCOMPLETE;

    if (direction == CW) /* Spin Clockwise */
    {
      WHL_L1 = -speed;
      WHL_R1 = speed;

    }else /* Spin the opposite direction - Counter Clockwise */
    {
      WHL_L1 = speed;
      WHL_R1 = -speed;
    }
  }
  else
  { // All done (move on to next drive command)
    Drive_Status = COMPLETE;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    PUSH_DOWN_PADS = 0;
    PUSH_DOWN_PADS_REV = 0;
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;
  }
}


/*******************************************************************************
* FUNCTION NAME: Drive_Forward_Dist
* PURPOSE:       Simple drive straight function for function-based autonomous mode.
*                Top speed @ full weight on carpet is 9.38 fps
*
*    Figure out how to reliably dynamically detect the slower side each time
*******************************************************************************/
void Drive_Forward_Dist(int dist, int speed)
{
  static signed long desired_distance; // in encoder counts
  signed long encoder_dist;
  int PID;
  static unsigned long LastClock=0;
  static int LastActual, CumError, desired_speedL, desired_speedR;
//  static int , CumErrorR, CumErrorL;
  static char finishCount, EndGame;
  static char PID_left_side, PickSlowSide;
  static signed long slowDown;


  if (lastDriveCmd != DISTFWD)
  {
    lastDriveCmd = DISTFWD;

    desired_distance = -dist;
    /* Encoder has 256 counts per revolution and travels 16.28 inches per revolution */
    desired_distance = desired_distance * (long)TICKS_PER_REV / (long)ENCODER_INCHES; // Convert from inches to encoder ticks
    desired_speedR = desired_speedL = speed;
#ifdef _DEBUG
    PrintCR();
    PrintString((char *) " **Drive_Forward_Dist** , ");
    PrintSignedDecimal(dist); PrintString((char *) "in.");
    PrintString((char *) ", Ticks="); PrintSignedDecimal(desired_distance);
    PrintString((char *) ", speed="); PrintSignedDecimal(speed);
#endif
    /* Disable interrupts while we zero the shaft encoders */
    INTCONbits.GIEL = 0;
    Right_Encoder_Count = 0;
    Left_Encoder_Count = 0;
    INTCONbits.GIEL = 1;

    PID = 0;
    LastActual = 0;
    CumError = 0;
//    CumErrorR = 0; CumErrorL = 0;
    finishCount = 0;
    EndGame = FALSE;
    PID_left_side = TRUE;
    PickSlowSide = FALSE;
    slowDown = (long)SLOWDOWN * (long)speed / (long)FULLSPEED; // Start braking proportional to speed
#ifdef _DEBUG
    PrintString((char *) ", slowDown="); PrintSignedDecimal(slowDown);
    PrintCR();
#endif
  }

  if (!PickSlowSide) // First encoder to register movement wins 
  {
    if (Right_Encoder_Count < (Left_Encoder_Count+2))
    {
      PickSlowSide = TRUE;
      PID_left_side = TRUE;
    }
    else if (Right_Encoder_Count > (Left_Encoder_Count-2))
    {
      PickSlowSide = TRUE;
      PID_left_side = FALSE;
    }
  }
  encoder_dist = (Right_Encoder_Count + Left_Encoder_Count) / 2;
#ifdef _DEBUG
    PrintString((char *) " Right="); PrintSignedDecimal(Right_Encoder_Count);
    PrintString((char *) ", Left="); PrintSignedDecimal(Left_Encoder_Count);
//    PrintString((char *) ", Clockms="); PrintDecimal(Clockms);
#endif

//  if ((desired_distance < (encoder_dist - (long)SLOWDOWN)) & !EndGame)
  if ((desired_distance < (encoder_dist - slowDown)) & !EndGame)
  {
    Drive_Status = INCOMPLETE;

    WHL_L1 = -desired_speedL;
    WHL_R1 = -desired_speedR;

//    CumErrorL += Right_Encoder_Count - Left_Encoder_Count;
//    CumErrorR += Left_Encoder_Count - Right_Encoder_Count;
    /* Balance the motors */
//    if (PID_left_side)  //Apply PID to the left side
//    {
//      PrintString((char *) ", CumErrorL="); PrintSignedDecimal(CumErrorL);
//      PID = Encoder_PID(Right_Encoder_Count, Left_Encoder_Count, &LastActual, &CumError);
//      WHL_L1 += PWMBalance(BalForLeft, desired_speedL) + PID;
////      WHL_R1 += PWMBalance(BalForRight, desired_speedR);
//      if (WHL_R1 < -127) WHL_R1 = -127;
//      if (WHL_L1 < -127) 
//      {
//        WHL_L1 = -127;
//        desired_speedL = 127;
//        desired_speedR = speed; // Make sure any prior adjustments don't carry over
//        CumError = -CumError;  // Need to examine results to see how this should be properly handled
//        CumError = 0; // LastActual isn't currently used
//        PID_left_side = FALSE;  // Max'ed out so start adjusting the other side
//      }
//      if (WHL_R1 > 0) WHL_R1 = 0;
//      if (WHL_L1 > 0) 
//      {
//        WHL_L1 = 0;
//        desired_speedR +=5; // Left side can't go any slower so speed up the right side
//      }
//    }
//    else  //Apply PID to the right side
//    {
//      PrintString((char *) ", CumErrorR="); PrintSignedDecimal(CumErrorR);
//      PID = Encoder_PID(Left_Encoder_Count, Right_Encoder_Count, &LastActual, &CumError);
////      WHL_L1 += PWMBalance(BalForLeft, desired_speedL);
//      WHL_R1 += PWMBalance(BalForRight, desired_speedR) + PID;
//  PrintString((char *) ", WHL_R="); PrintSignedDecimal(WHL_R1);
//  PrintString((char *) ", PID="); PrintSignedDecimal(PID);
//      if (WHL_L1 < -127) WHL_L1 = -127;
//      if (WHL_R1 < -127)
//      {
//        WHL_R1 = -127;
//        desired_speedR = 127;
//        desired_speedL = speed; // Make sure any prior adjustments don't carry over
//        CumError = -CumError;
//        CumError = 0;
//        PID_left_side = TRUE;  // Max'ed out so start adjusting the other side
//      }
//      if (WHL_R1 > 0) WHL_R1 = 0;
//      if (WHL_L1 > 0) WHL_L1 = 0;
//    }
    if (!PickSlowSide)
    {
      WHL_L1 += PWMBalance(BalForLeft, desired_speedL);
      WHL_R1 += PWMBalance(BalForRight, desired_speedR);
    }
    else
    {
      if (PID_left_side)  //Apply PID to the left side
      {
        PID = Encoder_PID(Right_Encoder_Count, Left_Encoder_Count, &LastActual, &CumError);

        /* Balance the motors */
        WHL_L1 += PWMBalance(BalForLeft, desired_speedL) + PID;
//        WHL_R1 += PWMBalance(BalForRight, desired_speedR);
        if (WHL_L1 > 0) WHL_L1 = 0;
        if (WHL_L1 < -127) WHL_L1 = -127;

        // If max PWM is reached by the adjusted side, then drop power on the opposite side
        if (WHL_L1 == -127)
        {
          desired_speedR -= 10;  // Could decrement by the amount of overshoot requested by PID
          WHL_R1 -= 10;
        }
        if (WHL_L1 == 0)
        {
          desired_speedR += 10;
          WHL_R1 += 10;
        }
        if (WHL_R1 > 0) WHL_R1 = 0;
        if (WHL_R1 < -127) WHL_R1 = -127;
      } 
      else
      {
        PID = Encoder_PID(Left_Encoder_Count, Right_Encoder_Count, &LastActual, &CumError);

        /* Balance the motors */
//        WHL_L1 += PWMBalance(BalForLeft, desired_speedL);
        WHL_R1 += PWMBalance(BalForRight, desired_speedR) + PID;
        if (WHL_R1 > 0) WHL_R1 = 0;
        if (WHL_R1 < -127) WHL_R1 = -127;

        // If max PWM is reached by the adjusted side, then drop power on the opposite side
        if (WHL_R1 == -127)
        {
          desired_speedL -= 10;  // Could decrement by the amount of overshoot requested by PID
          WHL_L1 -= 10;
        }
        if (WHL_R1 == 0)
        {
          desired_speedL += 10;
          WHL_L1 += 10;
        }
        if (WHL_L1 > 0) WHL_L1 = 0;
        if (WHL_L1 < -127) WHL_L1 = -127;
      }
    }
#ifdef _DEBUG
      PrintString((char *) ", PID="); PrintSignedDecimal(PID);
      PrintString((char *) ", Cum="); PrintSignedDecimal(CumError);
#endif
  }
  else if (finishCount < 5)  // Make sure the robot has settled at the desired position
  {
      if (finishCount == 0)
      {
        EndGame = TRUE;
        Brake((int)TRUE, desired_distance, desired_distance);
        finishCount++;
      }
      else
      {
          Brake((int)FALSE, desired_distance, desired_distance);
      }
      /* Can take average of encoders - It will end cockeyed */
      if ( (((Right_Encoder_Count + Left_Encoder_Count)/2) < (desired_distance+MINWINDOW)) & 
           (((Right_Encoder_Count + Left_Encoder_Count)/2) > (desired_distance-MINWINDOW)) )
      /* Can insist both encoders match but robot can shimmy a lot */      
//      if ((Right_Encoder_Count < (desired_distance+MINWINDOW)) & 
//          (Right_Encoder_Count > (desired_distance-MINWINDOW)) & 
//          (Left_Encoder_Count < (desired_distance+MINWINDOW)) & 
//          (Left_Encoder_Count > (desired_distance-MINWINDOW)))
      {
        finishCount++;
      }
  }
  else
  {
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    Drive_Status = COMPLETE;
  }
#ifdef _DEBUG
  PrintString((char *) ", WHL_R="); PrintSignedDecimal(WHL_R1);
  PrintString((char *) ", WHL_L="); PrintSignedDecimal(WHL_L1);
  PrintCR();
#endif
}

/*******************************************************************************
* FUNCTION NAME: Drive_Backward_Dist
* PURPOSE:       Simple drive straight function for function-based autonomous mode.
*                Top speed @ full weight on carpet is 9.38 fps
*******************************************************************************/
void Drive_Backward_Dist(int dist, int speed)
{
  static signed long desired_distance; // in encoder counts
  signed long encoder_dist;
  int PID;
  static int LastActual, CumError, desired_speedL, desired_speedR;
  static char finishCount, EndGame;
  static unsigned long LastClock=0;

  if (lastDriveCmd != DISTBCK)
  {
    lastDriveCmd = DISTBCK;

    desired_distance = dist;
    /* Encoder has 256 counts per revolution and travels 16.28 inches per revolution */
    desired_distance = desired_distance * (long)TICKS_PER_REV / (long)ENCODER_INCHES; // Convert from inches to encoder ticks
    desired_speedR = desired_speedL = speed;

#ifdef _DEBUG
    PrintCR();
    PrintString((char *) " **Drive_Backward_Dist** , ");
    PrintSignedDecimal(dist); PrintString((char *) "in.");
    PrintString((char *) ", Ticks="); PrintSignedDecimal(desired_distance);
    PrintCR();
#endif

    /* Disable interrupts while we zero the shaft encoders */
    INTCONbits.GIEL = 0;
    Right_Encoder_Count = 0;
    Left_Encoder_Count = 0;
    INTCONbits.GIEL = 1;
    
    PID = 0;
    LastActual = 0;
    CumError = 0;
    finishCount = 0;
    EndGame = FALSE;
  }

  encoder_dist = (Right_Encoder_Count + Left_Encoder_Count) / 2;

#ifdef _DEBUG
//    PrintString((char *) " Right="); PrintSignedDecimal(Right_Encoder_Count);
//    PrintString((char *) ", Left="); PrintSignedDecimal(Left_Encoder_Count);
//    PrintString((char *) ", Clockms="); PrintDecimal(Clockms);
#endif

  if ((desired_distance > (encoder_dist + (long)SLOWDOWN)) & !EndGame)
  {
    Drive_Status = INCOMPLETE;

    WHL_L1 = desired_speedL;
    WHL_R1 = desired_speedR;

    PID = Encoder_PID(Left_Encoder_Count, Right_Encoder_Count, &LastActual, &CumError);

    /* Balance the motors */
//    WHL_L1 -= PWMBalance(BalBckLeft, desired_speedL);
    WHL_R1 -= PWMBalance(BalBckRight, desired_speedR) - PID;
    if (WHL_R1 < 0) WHL_R1 = 0;
    if (WHL_R1 > 127) WHL_R1 = 127;
    // If max PWM is reached by the adjusted side, then drop power on the opposite side
    if (WHL_R1 == 127)
    {
      desired_speedL -=5;
      WHL_L1 -= 5;
    }
    else if (WHL_R1 == 0)
    {
      desired_speedL +=5;
      WHL_L1 += 5;
    }
    if (WHL_L1 < 0) WHL_L1 = 0;
    if (WHL_L1 > 127) WHL_L1 = 127;

#ifdef _DEBUG
      PrintString((char *) ", PID="); PrintSignedDecimal(PID);
      PrintString((char *) ", Cum="); PrintSignedDecimal(CumError);
      PrintString((char *) ", WHL_R="); PrintSignedDecimal(WHL_R1);
      PrintString((char *) ", WHL_L="); PrintSignedDecimal(WHL_L1);
      PrintCR();
#endif
  }
  else if (finishCount < 5)  // Make sure the robot has settled at the desired position
  {
      if (finishCount == 0)
      {
        EndGame = TRUE;
        Brake((int)TRUE, desired_distance, desired_distance);
        finishCount++;
      }
      else
      {
          Brake((int)FALSE, desired_distance, desired_distance);
      }
      /* Can take average of encoders - It will end cockeyed */
      if ( (((Right_Encoder_Count + Left_Encoder_Count)/2) < (desired_distance+MINWINDOW)) & 
           (((Right_Encoder_Count + Left_Encoder_Count)/2) > (desired_distance-MINWINDOW)) )
      /* Can insist both encoders match, but robot can shimmy */      
//      if ((Right_Encoder_Count < (desired_distance+MINWINDOW)) & 
//          (Right_Encoder_Count > (desired_distance-MINWINDOW)) & 
//          (Left_Encoder_Count < (desired_distance+MINWINDOW)) & 
//          (Left_Encoder_Count > (desired_distance-MINWINDOW)))
      {
        finishCount++;
      }
  }
  else
  {
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    Drive_Status = COMPLETE;
  }
#ifdef _DEBUG
  PrintString((char *) ", WHL_R="); PrintSignedDecimal(WHL_R1);
  PrintString((char *) ", WHL_L="); PrintSignedDecimal(WHL_L1);
  PrintCR();
#endif
}
