/*******************************************************************************
* FILE NAME: auto_misc_routines.c
*
*	Version 4.0	last updated 8/2/04::
*
* DESCRIPTION:
*  Autonomous miscellaneous functions
*
* USAGE:
*  Just use it...
*
*******************************************************************************/

#ifdef _DEBUG
#include "print_routines.h"
#endif

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"

#include "motor_routines.h"
#include "auto_routines.h"
#include "user_routines.h"
#include "clock_routines.h"
#include "sensor_routines.h"



/*******************************************************************************
* FUNCTION NAME: Goal_Grab
* PURPOSE:       Activate goal latch
*******************************************************************************/
//void Goal_Grab(int time)
//{
//  static unsigned long duration; //in tenths of seconds
//
//  if (lastDriveCmd != GOALGRAB)
//  {
//    lastDriveCmd = GOALGRAB;
//    duration = Clockds + time; //  Delay for one second to allow for drive coasting
//    Drive_Status = INCOMPLETE;
//#ifdef _DEBUG
//    PrintString((char *) " **Goal_Grab** ");
//    PrintCR();
//#endif
//  }
//
//  if (duration < Clockds)
//  {
//    PUSH_DOWN_PADS = 1;
//    relay4_rev = 0;  // Not required but should be set as a matter of principle
//    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
//    Drive_Status = COMPLETE;
//  }
//}


/*******************************************************************************
* FUNCTION NAME: Goal_Release
* PURPOSE:       Activate goal latch
*******************************************************************************/
//void Goal_Release(int time)
//{
//  static unsigned long duration; //in tenths of seconds
//
//  if (lastDriveCmd != GOALGRAB)
//  {
//    lastDriveCmd = GOALGRAB;
//    duration = Clockds + time; //  Delay for one second to allow for drive coasting
//    Drive_Status = INCOMPLETE;
//#ifdef _DEBUG
//    PrintString((char *) " **Goal_Release** ");
//    PrintCR();
//#endif
//  }
//
//  if (duration < Clockds)
//  {
//    PUSH_DOWN_PADS = 0;
//    relay4_rev = 0;
//    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
//    Drive_Status = COMPLETE;
//  }
//}


/*******************************************************************************
* FUNCTION NAME: Delay
* PURPOSE:       Delay for a period of time in tenths of seconds
*                There is an implied SYNC here. This will pause all operations.
*******************************************************************************/
void Delay(int time)
{
  static unsigned long duration;
  static char startFlag;

  if (lastJointCmd != DELAY)
  {
    duration = (unsigned long) time;
    startFlag=TRUE;
    lastJointCmd = DELAY;
    Joint_Status = INCOMPLETE;
#ifdef _DEBUG
    PrintString((char *) " **Delay** ");
    PrintCR();
#endif
  }

  /* Don't start until both drive and arm command queues are free */
  if (Drive_Status == COMPLETE) 
    Drive_Status = JOINT;
  if (Arm_Status   == COMPLETE) 
    Arm_Status   = JOINT;

  if ((Drive_Status == JOINT) & (Arm_Status == JOINT))
  {
    if (startFlag)
    {
      startFlag = FALSE;
      duration = Clockds + duration; // Current time + duration = cmd completed
#ifdef _DEBUG
     PrintString((char *) "Delay duration = ");
     PrintDecimal(duration);
#endif
    }

    if (duration > Clockds)
    { // Keep waiting
      Joint_Status = INCOMPLETE;
    }
    else
    { // All done (move on to next drive command)
      Joint_Status = COMPLETE;
      Drive_Status = COMPLETE;
      Arm_Status   = COMPLETE;
      lastJointCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    }
  }
}


/*******************************************************************************
* FUNCTION NAME: Synch_Drive_Arm
* PURPOSE:       Synchronize the Arm and Drive commands
*                Wait for both to COMPLETE, before proceeding
*******************************************************************************/
void Sync_Drive_Arm(int time)
{
  static unsigned long duration;
  static char startFlag;

  if (lastJointCmd != SYNC)
  {
    startFlag=TRUE;
    lastJointCmd = SYNC;
    Joint_Status = INCOMPLETE;
#ifdef _DEBUG
    PrintString((char *) " **Synch_Drive_Arm** ");
    PrintCR();
#endif
  }

  /* Don't start until both drive and arm command queues are free */
  if (Drive_Status == COMPLETE) Drive_Status = JOINT;
  if (Arm_Status   == COMPLETE) Arm_Status   = JOINT;

  if ((Drive_Status == JOINT) & (Arm_Status == JOINT))
  {
    if (startFlag)
    {
      startFlag = FALSE;
      duration = Clockds + (unsigned long) time; // Delay to allow previous operations to settle
    }

    if (duration > Clockds)
    { // Delay to allow previous operations to settle
      Joint_Status = INCOMPLETE;
    }
    else
    {
      Joint_Status = COMPLETE;
      Drive_Status = COMPLETE;
      Arm_Status   = COMPLETE;
    }
  }
}
