/*******************************************************************************
* FILE NAME: auto_routines.c
*
*	Version 4.0	last updated 8/2/04::
*
* DESCRIPTION:
*  this is the autonomous mode place
*   this is where the autonomous mode is places
*
* USAGE:
*  Just use it...
*
*******************************************************************************/

#ifdef _DEBUG
#include "print_routines.h"
#endif

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"

#include "motor_routines.h"
#include "auto_routines.h"
#include "user_routines.h"
#include "clock_routines.h"
#include "sensor_routines.h"

#include "auto_scripts.h"  // All autonomous scripts reside here

/* Status is set by the individual auto functions */

/*******************************************************************************
* FUNCTION NAME: User_Autonomous_Code
* PURPOSE:       this is called constantly and checks to see if it is time to
*				  execute a function. (This is Autonomous mode)
* CALLED FROM:   main.c
* ARGUMENTS:     void
* RETURNS:       moves the robot based on how it is set.
*******************************************************************************/
void User_Autonomous_Code(void)
{
  static unsigned int swtc, sw16;  //set equal to the switch input

  /** Autonomous is selected by the auto mode selector **/
  /* Only check this switch once at the start */
  if (AutoFirstTime)
  { /* Initialize the first time autonomous starts */
    AutoFirstTime = FALSE;
    sw16 = swtc = PORTJ>>4;  // Auto select switch 1-16
//    PrintString((char *) " sw16 = ");
//    PrintDecimal (sw16);

//    PUSH_DOWN_PADS = 0; // Make sure push down pads are up
//    relay3_rev = 0;
    AutoGoal = FALSE; // Leave goal in position until first user trigger
    holdflag = FALSE;
    holdarm = FALSE;

    Drive_Status=COMPLETE; Arm_Status=COMPLETE; Joint_Status=COMPLETE;
    lastDriveCmd=ENDCMD;   lastArmCmd=ENDCMD;   lastJointCmd=ENDCMD;
    DriveCmd=0;            ArmCmd=0;            JointCmd=0; // Current cmd # in script
    cmdNum=0; lastcmdNum=99;
    
    ARM_PWM = ARMNEUTRAL;
    TELESCOPE_PWM1 = NEUTRAL;
    WHL_R1 = WHL_R2 = NEUTRAL;
    WHL_L1 = WHL_L2 = NEUTRAL;

  } /* End FirstTime */

//  The FIELD position switch is used within certain functions to reverse script 
//  cmd parameters mirroring operations on the opposite side of the field
//  Commands affected include: DSPIN, DISTTURN, GTURN, GTURNGOAL

//  if(sw16 == 9)
//    RedBlue == BLUE;
//  else
//    RedBlue == RED;

  {
    if (rc_dig_in11 == 0) //Right
    {
      if (sw16 == 9) // Red or Blue side of field
      {
PrintString((char *)"right - BLUE");
        swtc = 10; //Blue side of field
      }
      else if (sw16 == 15) //Keep it straight only
      {
        swtc = 12; // Stay and play in our zone
      }
      else
      {
PrintString((char *)"right - RED");
        swtc = 11; //RED side of field
      }
//      swtc = 14;
    }


    else if (rc_dig_in12 == 0)  //Left
    {
//PrintString((char *)"left");
      if (sw16 == 9) // Red or Blue side of field
      {
PrintString((char *)"left - BLUE");
        swtc = 0; //Blue side of field
      }
      else if (sw16 == 15) //Keep it straight only
      {
        swtc = 2; // Stay and play in our zone
      }
      else
      {
PrintString((char *)"left - RED");
        swtc = 1; //RED side of field
      }
//      swtc = 0;
    }


    else if (rc_dig_in13 == 0) //Center
    {
//PrintString((char *)"center");
      if (sw16 == 9) // Red or Blue side of field
      {
PrintString((char *)"center - BLUE");
        swtc = 5; //Blue side of field
      }
      else if (sw16 == 15) //Keep it straight only
      {
        swtc = 7; // Stay and play in our zone
      }
      else
      {
PrintString((char *)"center - RED");
        swtc = 6; //RED side of field
      }
    }
  }


/*************** DEBUG OVERRIDE ****************/
//      swtc = 3;
/***********************************************/

    /* Copy selected script into workspace */
    switch(swtc)
	  {
		  case 0: 
	    default:
        Switch_Auto(AUTO_0);
			  break;
  		case 1: 
        Switch_Auto(AUTO_1);
		  	break;
  		case 2: 
        Switch_Auto(AUTO_2);
		  	break;
  		case 3: 
        Switch_Auto(AUTO_3);
		  	break;
  		case 4: 
            Switch_Auto(AUTO_4);
  	  	    break;
  		case 5: 
        Switch_Auto(AUTO_5);
		  	break;
  		case 6: 
        Switch_Auto(AUTO_6);
		  	break;
  		case 7: 
        Switch_Auto(AUTO_7);
		  	break;
  		case 8: 
        Switch_Auto(AUTO_8);
		  	break;
  		case 9: 
        Switch_Auto(AUTO_9);
		  	break;
  		case 10: 
        Switch_Auto(AUTO_10);
		  	break;
  		case 11: 
        Switch_Auto(AUTO_11);
		  	break;
  		case 12: 
        Switch_Auto(AUTO_12);
		  	break;
  		case 13: 
        Switch_Auto(AUTO_13);
		  	break;
  		case 14: 
        Switch_Auto(AUTO_14);
		  	break;
  		case 15: 
        Switch_Auto(AUTO_15);
		  	break;
	   }
  /* 4 motor drive */
  WHL_L2 = WHL_L1;	//set the other motor to same
  WHL_R2 = WHL_R1;  //same note as above...
}


/*******************************************************************************
* FUNCTION NAME: Switch_Auto
* PURPOSE:       Process an autonomous command script
* ARGUMENTS:     Command script
* RETURNS:       moves the robot based on the command script.
*******************************************************************************/
void Switch_Auto(rom const int *CommandScript)
{
  int speed, distance, time, direction, heading, distance_right, distance_left, faster;
  int setting, nextCmd, unused;
  int i, j, k;
  static unsigned long LastClock=0;

  /* Process existing commands - */
  /* Original command arguments are preserved within the functions themselves */

  /*******************************/
  /************ Drive ************/
  switch(lastDriveCmd)
  {
      default:  // includes JOINT cmds
      case ENDCMD:  // Do nothing - no drive cmds yet
          if (holdflag)
          {
            Hold_Position((int)FALSE, (int)0, (int)0); // Hold this position even if pushed
          }
          else
          { // Initialize
            Hold_Position((int)TRUE, (int)0, (int)0);

            /* Disable interrupts while we zero the shaft encoders */
            INTCONbits.GIEL = 0;
            Right_Encoder_Count = 0;
            Left_Encoder_Count = 0;
            INTCONbits.GIEL = 1;
            holdflag = TRUE;
          }
          break;    
      case DFWD:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Forward_Time(time, speed);
          break;
      case DBCK:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Backward_Time(time, speed);
          break;
        case DSPIN:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          direction = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIRECTION];
          Drive_Spin_In_Place_Time(time, speed, direction);
          break;
      case DISTFWD:
          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIST];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Forward_Dist(distance, speed);
          break;
      case DISTBCK:
          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIST];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Backward_Dist(distance, speed);
          break;
//      case GOALGRAB:
//          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
//          Goal_Grab(time);
//          break;
//      case GOALREL:
//          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
//          Goal_Release(time);
//          break;
      case GTURN:
          heading = CommandScript[DriveCmd*AUTO_WIDTH + CMDGHEADING];
          Gyro_Turn(heading);
          break;
//      case FBALANCE:
//          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIST];
//          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
//          BalanceF(distance, speed);
//          break;
//      case RBALANCE:
//          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIST];
//          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
//          BalanceR(distance, speed);
//          break;/*
//      case BALANCE:
//          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIST];
//          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
//          faster = CommandScript[DriveCmd*AUTO_WIDTH + CMDFASTER];
//          Balance(distance, speed, faster);*/
//          break;
  }
  /*******************************/
  /************** Arm   **********/
  switch(lastArmCmd)
  {
     default:
     case ENDCMD:  // Do nothing - no arm cmds yet
          break;    
      case ARMOPN:
          Arm_Open();
          break;
      case ARMCLS:
          Arm_Close();
          break;
      case ARMFWD:
          time = CommandScript[ArmCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[ArmCmd*AUTO_WIDTH + CMDSPEED];
          Arm_Forward_Time(time, speed);
          break;
      case ARMBCK:
          time = CommandScript[ArmCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[ArmCmd*AUTO_WIDTH + CMDSPEED];
          Arm_Backward_Time(time, speed);
          break;
      case ARMPVT:
          setting = CommandScript[ArmCmd*AUTO_WIDTH + CMDSETTING];
          Arm_Pivot(setting);
          break;
      case ARMTEL:
          setting = CommandScript[ArmCmd*AUTO_WIDTH + CMDSETTING];
          Arm_Telescope(setting);
          break;
//      case ARMWAV:
//          time = CommandScript[ArmCmd*AUTO_WIDTH + CMDTIME];
//          speed = CommandScript[ArmCmd*AUTO_WIDTH + CMDSPEED];
//          Arm_Wave(time, speed);
//          break;
  }
  if (holdarm) // Arm is supposed to be stationary
  {
    Hold_Arm_Position(FALSE, unused, &ARM_PWM);
  }

  /*******************************/
  /************* Joint ***********/
  switch(lastJointCmd)
  {
      default:
      case ENDCMD:  // Do nothing - no joint cmds yet
          break;    
      case DELAY:
          time = CommandScript[JointCmd*AUTO_WIDTH + CMDTIME];
          Delay(time);
          break;
      case SYNC:
          time = CommandScript[JointCmd*AUTO_WIDTH + CMDTIME];
          Sync_Drive_Arm(time);
          break;
  }

/******************************/

  /* Look at next command on each queue */
  nextCmd = CommandScript[cmdNum*AUTO_WIDTH + CMD];

  if (nextCmd != ENDCMD)
  {
   if (nextCmd <= GTURN) // It's a Drive command
   {
     if (Drive_Status == COMPLETE)
    {    /* Start next Drive cmd */
      DriveCmd = cmdNum;
      cmdNum++;  // Get ready for next command
      holdflag = FALSE;
      switch(nextCmd)
      { 
        default:
        case ENDCMD:  // Do nothing - no drive cmds yet
          break;    
        case DFWD:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Forward_Time(time, speed);
          break;
        case DBCK:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Backward_Time(time, speed);
          break;
        case DSPIN:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          direction = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIRECTION];
          Drive_Spin_In_Place_Time(time, speed, direction);
          break;
        case DISTFWD:
          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIST];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Forward_Dist(distance, speed);
          break;
        case DISTBCK:
          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIST];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Backward_Dist(distance, speed);
          break;
//        case GOALGRAB:
//          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
//          Goal_Grab(time);
//          break;
//        case GOALREL:
//          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
//          Goal_Release(time);
//          break;
        case GTURN:
          heading = CommandScript[DriveCmd*AUTO_WIDTH + CMDGHEADING];
          Gyro_Turn(heading);
          break;
//      case FBALANCE:
//          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIST];
//          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
//          BalanceF(distance, speed);
//          break;
//      case RBALANCE:
//          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIST];
//          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
//          BalanceR(distance, speed);
//          break;/*
//      case BALANCE:
//          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIST];
//          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
//          faster = CommandScript[DriveCmd*AUTO_WIDTH + CMDFASTER];
//          Balance(distance, speed, faster);
//          break;*/
      }
     }
    }
    else if (nextCmd >= ARMOPN)
    {
     if (Arm_Status == COMPLETE)
     {    /* Start next Arm cmd */
      ArmCmd = cmdNum;
      cmdNum++;  // Get ready for next command
      switch(nextCmd)
      {
        default:
        case ENDCMD:  // Do nothing - no arm cmds yet
          break;    
        case ARMOPN:
          Arm_Open();
          break;
        case ARMCLS:
          Arm_Close();
          break;
        case ARMFWD:
          time = CommandScript[ArmCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[ArmCmd*AUTO_WIDTH + CMDSPEED];
          Arm_Forward_Time(time, speed);
          break;
        case ARMBCK:
          time = CommandScript[ArmCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[ArmCmd*AUTO_WIDTH + CMDSPEED];
          Arm_Backward_Time(time, speed);
          break;
        case ARMPVT:
          setting = CommandScript[ArmCmd*AUTO_WIDTH + CMDSETTING];
          Arm_Pivot(setting);
          break;
      case ARMTEL:
          setting = CommandScript[ArmCmd*AUTO_WIDTH + CMDSETTING];
          Arm_Telescope(setting);
          break;
//        case ARMWAV:
//          time = CommandScript[ArmCmd*AUTO_WIDTH + CMDTIME];
//          speed = CommandScript[ArmCmd*AUTO_WIDTH + CMDSPEED];
//          Arm_Wave(time, speed);
//          break;
      }
     }
    }
    else if (Joint_Status == COMPLETE)
    {    /* Start next Joint cmd */
      JointCmd = cmdNum;
      cmdNum++;  // Get ready for next command
      switch(nextCmd)
      {
        default:
        case ENDCMD:  // Do nothing - no joint cmds yet
          break;    
        case DELAY:
          time = CommandScript[JointCmd*AUTO_WIDTH + CMDTIME];
          Delay(time);
          break;
        case SYNC:
          time = CommandScript[JointCmd*AUTO_WIDTH + CMDTIME];
          Sync_Drive_Arm(time);
          break;
      }
    }
  } /* End ENDCMD check */
}
