/*******************************************************************************
* FILE NAME: auto_routines.h
*
*	Version 3.13	last updated 2/21/04::
*
* DESCRIPTION:
*  This file contains several sample autonomous techniques as a
*  progressive development from prinitive to more sophisticated.
*    
*
* USAGE:
*
*******************************************************************************/

#ifndef __auto_routines_h_
#define __auto_routines_h_

#ifdef VARIABLE_DECLARE
   char Arm_Status, Drive_Status, Joint_Status;
   int lastDriveCmd, lastArmCmd, lastJointCmd;
   unsigned char DriveCmd, ArmCmd, JointCmd; // Current cmd # in script
   unsigned char cmdNum,lastcmdNum;
   char holdflag, holdarm;
   char RedBlue;
#else
  extern char Arm_Status, Drive_Status, Joint_Status;
  extern int lastDriveCmd, lastArmCmd, lastJointCmd;
  extern unsigned char DriveCmd, ArmCmd, JointCmd; // Current cmd # in script
  extern unsigned char cmdNum,lastcmdNum;
  extern char holdflag, holdarm;
  extern char RedBlue;
#endif

#define BLUE 0
#define RED  1

/**** Script Commands ****/
  #define ENDCMD        0   /* Must always be the last command */

  /* Driving commands */
  #define DFWD          1   /* Drive Forward for a time */
  #define DBCK          2   /* Drive Backward for a time */
  #define DSPIN         3   /* Spin in place for a time */
  #define DISTFWD       4   /* Drive forward a distance */
  #define DISTBCK       5   /* Drive backward a distance */
  #define DISTTURN      6   /* Turn in place a distance */
  #define DSTOP         7   /* Stop all engines */

  /* Miscellaneous driving commands */
  #define GOALGRAB	    8   /* Goal grabber */
  #define GOALREL 	    9   /* Goal grabber release */
  #define IRTURN	      10	/* Track the IR Beacon - operates like GTURN */
  #define CURVE		    	11	/* For curving around(superfast[fastest possible]) */
  #define GOALSRC	      12   /* Search for Goal then grab */
  /* Diagnostic commands */
//  #define FBALANCE      13  /* DIAGNOSTIC - Measure the drivetrain balance #'s */
//  #define RBALANCE      14  /* DIAGNOSTIC - Measure the drivetrain balance #'s */
//  #define BALANCE       15  /* DIAGNOSTIC - Measure the drivetrain balance #'s */
  /* Add new drive cmds here */
  #define GTURNWGOAL    18  /* Gyro controlled turn with heavier goal */
  #define GTURN         19  /* Gyro controlled turn */

  /* Joint commands - requires both arm and drive queues to complete */
  #define DELAY   	    20  /* Sit still for a time */
  /* Add new joint cmds here */
  #define SYNC          29  /* Wait for both arm and drive cmds to complete before proceeding */

  /* Arm commands */
  #define ARMOPN		30   /* Arms open */
  #define ARMCLS		31   /* Arms close */
  #define ARMFWD		32  /* Arms Forward for a time */
  #define ARMBCK		33  /* Arms Backward for a time */
  #define ARMPVT		34  /* Arms to a specified position setting */
  #define ARMSTP		35  /* Arms Stop */
  #define ARMTEL		36  /* Arm telescope */
  /* Add new arm cmds here */

  /******** End Script Commands *********/

  /* command array indexes - cmd parameters depend on the cmd */
  #define CMD           0

  #define CMDTIME       1
  #define CMDDIST       1
  #define CMDDISTRIGHT  1
  #define CMDGHEADING   1
  #define CMDIRHEADING  1
  #define CMDSETTING    1

  #define CMDSPEED      2
  #define CMDDISTLEFT   2

  #define CMDDIRECTION  3
  #define CMDFASTER     3

  /*************************/

/* Direction, speed */
#define CW               1  // Clockwise
#define CCW              0  // Counter clockwise
#define NONE             0
#define NEUTRAL          0
#define FULLSPEED      127 
#define PIDSPEED       100  // Max to allow PID control leaway
#define HALFSPEED       64
#define QTRSPEED        32
#define SLOWDOWN        500 // Encoder distance at which to start slowing down (500=21 in.)
#define SLOWDOWNGYRO    10  // Gyro degrees from target at which to start slowing down
#define MINTURNSPEED    70  // Minimum speed in autonomous turn (was 50)
#define MINSPEED        30  // Minimum speed driving straight (25 is absolute min)
#define MINWINDOW       20   //Target encoder window considered success (5 is too small,20 is okay)

#define MAXTIME			65535

#define CIRC_10		   111			//circumference times 11.05 in inches
#define DISTCALLS	   256			//number of times shaft encoder called per circ 

/* Command queue status */
#define COMPLETE     0
#define INCOMPLETE   1
#define JOINT        2

/* Balancing flags */
#define BalForLeft  1
#define BalForRight 2
#define BalBckLeft  3
#define BalBckRight 4

#define RIGHTSIDE   1
#define LEFTSIDE    2

/* This is the autonomous mode switched to.... this is always called in autonomous... */

void Switch_Auto(rom const int *);

/*Driving functions */
void Drive_Forward_Time(int,int);
void Drive_Backward_Time(int,int);
void Drive_Spin_In_Place_Time(int, int, int);
void Drive_Forward_Dist(int,int);
void Drive_Backward_Dist(int,int);
void Drive_Spin_In_Place_Dist(int, int, int);
void Drive_Curve(int, int, int);
void Gyro_Turn(int);
void Gyro_Turn_Goal(int);
void Drive_Stop(void);
//void BalanceF(int,int);
//void BalanceR(int,int);
//void Balance(int,int,int);

/* Goal functions */
//void Goal_Release(int);
//void Goal_Grab(int);
//void Goal_Search(int);

/* Arm functions */
void Arm_Open(void);
void Arm_Close(void);
void Arm_Forward_Time(int, int);
void Arm_Backward_Time(int, int);
void Arm_Pivot(int);
void Arm_Telescope(int);
void Arm_Stop(void);
//void Arm_Wave(int, int);

/* IR beacon functions */
//void SwitchIR_Auto(int, int, int);
//void IR_Beacon_Move(int, int);
//void IR_Homing(void);
//void IR_Turn(int);


/* Miscellaneous */
void Delay(int);
void Sync_Drive_Arm(int);

#endif
