/*******************************************************************************
* FILE NAME: auto_scripts.c
*
*	Version 5.0	last updated 3/17/04::
*
* DESCRIPTION:
*  These are all the autonomous command scripts
*  The auto script is selected via the 16 position auto mode switch on the robot and
*  the left/right of the field switch.
*  The left/right scripts are not simply reversed because the mobile goal and it's
*  utility to our alliance changes with the side of the field.
*
* USAGE:
*  Modify scripts as required
*  Cut & Paste command templates located at the bottom of this file
*
*  Drive commands must be intermixed with Arm commands carefully.
*  Two of either type in a row will block further command processing
*  until they have cleared the queue.
*
* Avoid running motors at full speed, so PID has room to adjust with
*
*******************************************************************************/

/****************************************************************************
 *                                                                          *
 *     This is where we load the different versions of the autonomous       *
 *                                                                          *
 ****************************************************************************/

#define AUTO_HEIGHT		20  // Maximum number of script commands
#define AUTO_WIDTH		4   // Maximum number of command parameters

/*-----------------------------------------------------------------------------*/
/*------------------EACH IS SPECIFIC TO ALLIANCE SIDE OF FIELD-----------------*/
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_0[AUTO_HEIGHT*AUTO_WIDTH] = 
  {/* 2005 left - BLUE side of field */
   DISTFWD,  48,  70,        0,  // Drive Forward  (distance,speed)
   ARMPVT,  901,  127,        0,  // was 892  
   SYNC,     0,        0,        0,
   GTURN,   -110,   0,        0,  // Turn to a heading (gyro)
   ARMPVT,  927,  127,        0,  // was 918   
   GTURN,  0,   0,        0,  // Turn to a heading (gyro)
   DISTFWD, 156,  70,        0,  // Drive Forward  (distance,speed) //was 70
   GTURN,  -90,   0,        0,  // Turn to a heading (gyro)
   ARMPVT,  877,  127,        0,  // was 868  
//Out of time
//   SYNC,     0,        0,        0,
   DISTFWD,   30,    70,        0,
   ENDCMD, NONE,  NONE,       NONE     //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_1[AUTO_HEIGHT*AUTO_WIDTH] = 
  {/* 2005 left - RED side of field */
   DISTFWD,  48,  70,        0,  // Drive Forward  (distance,speed)
   ARMPVT,  901,  127,        0,  // Was 876   
   GTURN,   -110,   0,        0,  // Turn to a heading (gyro)
   SYNC,     0,        0,        0,
   ARMPVT,  927,  127,        0,  // Arms to a specified position (setting,speed)   
   GTURN,  0,   0,        0,  // Turn to a heading (gyro)
   DISTFWD, 156,  70,        0,  // Drive Forward  (distance,speed) //was 70
   GTURN,  90,   0,        0,  // Turn to a heading (gyro)
   ARMPVT,  877,  127,        0,  // Arms to a specified position (setting,speed)   
//Out of time
//   SYNC,     0,        0,        0,
   DISTFWD,   130,    100,        0,
  ENDCMD, NONE,  NONE,       NONE     //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_2[AUTO_HEIGHT*AUTO_WIDTH] = 
  { /* 2005 left -straight only */
   DISTFWD,  48,  70,        0,  // Drive Forward  (distance,speed)
   ARMPVT,  901,  127,        0,  // Arms to a specified position (setting,speed)   
   GTURN,   -110,   0,        0,  // Turn to a heading (gyro)
   SYNC,     0,        0,        0,
   ARMPVT,  927,  127,        0,  // Arms to a specified position (setting,speed)   
   GTURN,  0,   0,        0,  // Turn to a heading (gyro)
   DISTFWD, 144,  70,        0,  // Drive Forward  (distance,speed) //192-48
//   GTURN,  90,   0,        0,  // Turn to a heading (gyro)
//   ARMPVT,  876,  127,        0,  // Arms to a specified position (setting,speed)   
//   DISTFWD, 13,  70,        0,  // Drive Forward  (distance,speed) //192-48
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_3[AUTO_HEIGHT*AUTO_WIDTH] = 
  { 
   DISTFWD,  152,   70,     0,  // Drive Forward  (distance,speed) was 140
   ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_4[AUTO_HEIGHT*AUTO_WIDTH] = 
  {
   DISTFWD,  152,   70,     0,  // Drive Forward  (distance,speed) was 140
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_5[AUTO_HEIGHT*AUTO_WIDTH] = 
  {/* 2005 center - BLUE side of field */
   DISTFWD,  152,   70,     0,  // Drive Forward  (distance,speed) was 140
   GTURN,  -90,   0,        0,  // -90 for Blue side, 90 for Red side -reversed in code
   ARMPVT,  877,  FULLSPEED,    0,  // Arms to a specified position (setting,speed)
   DISTFWD,  100,   70,     0,  // Drive Forward  (distance,speed) maybe 115
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_6[AUTO_HEIGHT*AUTO_WIDTH] = 
  {/* 2005 center - RED side of field */
   DISTFWD,  152,   70,     0,  // Drive Forward  (distance,speed) was 140
   GTURN,  90,   0,        0,  // -90 for Blue side, 90 for Red side -reversed in code
   ARMPVT,  877,  FULLSPEED,    0,  // Arms to a specified position (setting,speed)
   DISTFWD,  100,   70,     0,  // Drive Forward  (distance,speed) maybe 115
  ENDCMD, NONE,  NONE,       NONE    //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_7[AUTO_HEIGHT*AUTO_WIDTH] = 
  {/* 2005 center - straight only */
   DISTFWD,  176,   70,     0,  // Drive Forward  (distance,speed) was 152
//   GTURN,  90,   0,        0,  // -90 for Blue side, 90 for Red side -reversed in code
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_8[AUTO_HEIGHT*AUTO_WIDTH] = 
  {
   DISTFWD,  152,   70,     0,  // Drive Forward  (distance,speed) was 140
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_9[AUTO_HEIGHT*AUTO_WIDTH] = 
  {
   DISTFWD,  152,   70,     0,  // Drive Forward  (distance,speed) was 140
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_10[AUTO_HEIGHT*AUTO_WIDTH] = 
  {/* 2005 right BLUE side of field */
   DISTFWD,  48,  70,        0,  // Drive Forward  (distance,speed)
   ARMPVT,  901,  127,        0,  // Arms to a specified position (setting,speed)   
   GTURN,   110,   0,        0,  // Turn to a heading (gyro)
   SYNC,     0,        0,        0,
   ARMPVT,  927,  127,        0,  // Arms to a specified position (setting,speed)   
   GTURN,     0,   0,        0,  // Turn to a heading (gyro)
   DISTFWD, 156,  70,        0,  // Drive Forward  (distance,speed) //was 70
   ARMPVT,  877,  127,        0,  // Arms to a specified position (setting,speed)   
   GTURN,  -90,   0,        0,  // Turn to a heading (gyro)
//Out of time
//   SYNC,     0,        0,        0,
   DISTFWD,   130,    100,        0,
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_11[AUTO_HEIGHT*AUTO_WIDTH] = 
  {/* 2005 right RED side of field */
   DISTFWD,  48,  70,        0,  // Drive Forward  (distance,speed)
   ARMPVT,  901,  127,        0,  // Arms to a specified position (setting,speed)   
   GTURN,   110,   0,        0,  // Turn to a heading (gyro)
   SYNC,     0,        0,        0,
   ARMPVT,  927,  127,        0,  // Arms to a specified position (setting,speed)   
   GTURN,     0,   0,        0,  // Turn to a heading (gyro)
   DISTFWD, 156,  70,        0,  // Drive Forward  (distance,speed) //192-48
   ARMPVT,  877,  127,        0,  // Arms to a specified position (setting,speed)   
   GTURN,  90,   0,        0,  // Turn to a heading (gyro)
//Out of time
//   SYNC,     0,        0,        0,
   DISTFWD,   30,    100,        0,
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_12[AUTO_HEIGHT*AUTO_WIDTH] = 
  { /* 2005 right - straight only */
   DISTFWD,  48,  70,        0,  // Drive Forward  (distance,speed)
   ARMPVT,  901,  127,        0,  // Arms to a specified position (setting,speed)   
   GTURN,   110,   0,        0,  // Turn to a heading (gyro)
   SYNC,     0,        0,        0,
   ARMPVT,  927,  127,        0,  // Arms to a specified position (setting,speed)   
   GTURN,     0,   0,        0,  // Turn to a heading (gyro)
   DISTFWD, 156,  70,        0,  // Drive Forward  (distance,speed) //192-48
   ARMPVT,  877,  127,        0,  // Arms to a specified position (setting,speed)   
   GTURN,  90,   0,        0,  // Turn to a heading (gyro)
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_13[AUTO_HEIGHT*AUTO_WIDTH] = 
  { 
   DISTFWD,  152,   70,     0,  // Drive Forward  (distance,speed) was 140
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_14[AUTO_HEIGHT*AUTO_WIDTH] = 
  {/* test only */
   DISTFWD,  152,   70,     0,  // Drive Forward  (distance,speed) was 140
   ENDCMD, NONE,  NONE,       NONE     //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_15[AUTO_HEIGHT*AUTO_WIDTH] = 
  {/* 2005 center - Carry & cap tetra */  /****** DO NOT USE *************/
//   DISTFWD,  152,   70,     0,  // Drive Forward  (distance,speed) was 140
//   GTURN,  -90,   0,        0,  // -90 for Blue side, 90 for Red side -reversed in code
//   ARMPVT,  876,  FULLSPEED,    0,  // Arms to a specified position (setting,speed)
//   DISTFWD,  100,   70,     0,  // Drive Forward  (distance,speed) maybe 115

//   DISTFWD,  132,   70,     0,  // Drive Forward  (distance,speed)   
//   GTURN,  90,   0,        0,  // Turn to a heading (gyro)
//   DISTFWD,  110,   60,     0,  // Drive Forward  (distance,speed)
//   DISTBCK,  36,   60,     0,  // Drive Backward (distance,speed)

   ARMPVT,   889,   65,        0,  // Arms to a specified position (setting,speed)   
   ARMPVT,   902,   65,        0,  // Arms to a specified position (setting,speed)   
   ARMTEL,   400,    0,        0,  // Telescope arm to setting
   ARMPVT,   852,   65,        0,  // Arms to a specified position (setting,speed)   
   SYNC,     0,        0,        0,
   DISTFWD,  152,   60,        0,  // Drive Forward  (distance,speed) was 140
   SYNC,     0,        0,        0,
   ARMPVT,   862,   65,        0,  // Arms to a specified position (setting,speed)   
   DISTBCK,   20,   50,        0,
   GTURN,     90,    0,        0,  // Turn to a heading (gyro)   
   DISTFWD,   70,   60,        0,  // Drive Forward  (distance,speed) was 140
   ENDCMD,  NONE, NONE,     NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/

  /******** End autonomous commands ********/
  /*****************************************/
  /*****************************************/

/*******************************************************************************
* TIME is in tenths of seconds, speed is 0-127
* Available Commands:
*   !!!!!!!!! This must ALWAYS be the last command in a script !!!!!!!!!!
   ENDCMD,      0,           0,     0,   // End of the Command script
*
* Driving :
*(distance) - Encoder based, DIST is in inches
   DISTFWD,  DIST,   PIDSPEED,     0,  // Drive Forward  (distance(in),speed)
   DISTBCK,  DIST,   PIDSPEED,     0,  // Drive Backward (distance(in),speed)
*(timed) - 87"/sec startup, 112.5"/sec from then on
   DFWD,     TIME,   FULLSPEED,     0,  // Drive Forward (time,speed)
   DSPIN,    TIME,   FULLSPEED,    CW,  // Spin (time, speed, CW or CCW)
   DBCK,     TIME,   FULLSPEED,     0,  // Drive Backward (time,speed)
* (sensor based)
   GTURN,  GYROHEADING,   0,        0,  // Turn to a heading (degrees)
* (other)
   GOALGRAB,    TIME,     0,        0,  // Grab a goal (time delay)
   GOALREL,     TIME,     0,        0,  // Release a goal (time delay)
*
* ARMS:
   ARMOPN,     0,         0,        0,  // Open the 2x ball arms (no args)
   ARMCLS,     0,         0,        0,  // Close the 2x ball arms (no args)
*(timed)
   ARMFWD,   TIME,    FULLSPEED,    0,  // Arms forward (time,speed) 1.2sec=28"(10pt ball),2.2sec=58"(2X ball on mobile goal)
   ARMBCK,   TIME,    FULLSPEED,    0,  // Arms backward (time,speed)
   ARMWAV,   TIME,    FULLSPEED,    0,  // Arms Wave "Hello"
   ARMWAV,      5,     QTRSPEED,    0,  // Arms Wave "Hello"
   ARMTEL,SETTING,            0,    0,  // Telescope arm to setting
*(sensor)
   ARMPVT,  SETTING,  FULLSPEED,    0,  // Arms to a specified position (setting,speed)
*
* JOINT:
   DELAY,    TIME,        0,        0,  // Sit still (time)
   SYNC,     TIME,        0,        0,  // Wait for both Drive and Arm cmds to complete before proceeding (time)
*
* FUTURE COMMANDS:
   CURVE, DISTRIGHT, FULLSPEED, DISTLEFT,  // Follow a curved path ()
*
* BROKEN COMMANDS:
*
* DIAGNOSTIC COMMANDS:
   FBALANCE,  DIST,    FULLSPEED,     0,  // Measure the motor left/right PWM balance (time,speed)
   RBALANCE,  DIST,    FULLSPEED,     0,  // Measure the motor left/right balance (time,speed)
   BALANCE    DIST,    FULLSPEED,     fasterSIDE,  // Auto balance motors (fasterSIDE=RIGHTSIDE)
* PARAMETERS:
*  TIME: 1-150 tenths of seconds
*  Distance: 1-576 inches
*  Direction: CW or CCW
*  Speed: 1-127
*  Arm Setting: 1-30
*  Gyro heading: -360 to 360 & beyond, 0 is the original robot heading straight forward/backward
*  IR heading: -127 to 127 servo position 0 is straight backward
*  Arm timed moves- 1sec=25",2sec=53",3sec=85",4sec=109",7sec=2Xball,8sec=19"
*                   1.1sec=25",1.2sec=28",1.3sec=33",2.2sec=58"
*                   1.2sec=28"(10pt ball-bkwds)
*                   2.2sec=58"(2Xball-bkwds)
*                   7.0sec=2X ball frwds
*  DSPIN times:  2sec=260 degrees,  .8sec = 90 degrees
*******************************************************************************/
