/*******************************************************************************
* FILE NAME: main.c
*
* DESCRIPTION:
*  This file contains the main program loop.
*
* USAGE:
*  Note the different loop speed for the two routines:
*     Process_Data_From_Master_uP
*     Process_Data_From_Local_IO
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"

#include "user_routines.h"
#include "clock_routines.h"
#include "motor_routines.h"
#include "sensor_routines.h"

#ifdef _DEBUG
#include "print_routines.h"
#endif

tx_data_record txdata;          /* DO NOT CHANGE! */
rx_data_record rxdata;          /* DO NOT CHANGE! */
packed_struct statusflag;       /* DO NOT CHANGE! */

/*******************************************************************************
* FUNCTION NAME: main
* PURPOSE:       Main program function.
* CALLED FROM:   ifi_startup.c
* ARGUMENTS:     none
* RETURNS:       void
* DO NOT DELETE THIS FUNCTION 
*******************************************************************************/
#define STARTDELAY 38    // ~ One second at 26.2ms/loop
void main (void)
{
  /* Give the RC & OI systems time to yawn, stretch, have a little coffee - generally wake-up */
  /* Note sensors powered directly from 12v supply (eg Banners) may not initialize at the same time */
  static char warmUp= STARTDELAY;  // ~ One half second at 26.2ms/loop
  static long LastClock=0;

#ifdef UNCHANGEABLE_DEFINITION_AREA
  IFI_Initialization ();        /* DO NOT CHANGE! */
#endif

  User_Initialization();        /* You edit this in user_routines.c */
  AutoFirstTime=FALSE;    // Enable autonomous whenever the robot is disabled

  statusflag.NEW_SPI_DATA = 0;  /* DO NOT CHANGE! */ 

  while (1)   /* This loop will repeat indefinitely. */
  {
#ifdef _SIMULATOR
    statusflag.NEW_SPI_DATA = 1;
#endif

    if (statusflag.NEW_SPI_DATA)      /* 26.2ms loop area */
    {                                 /* I'm slow!  I only execute every 26.2ms because */
                                      /* that's how fast the Master uP gives me data. */
     Getdata(&rxdata);   /* Get fresh data from the master microprocessor. */
     if (warmUp > 0) // digital & OI inputs are suspect @ startup. Give them time to warm up
     {
        warmUp--;
     }
     else
     {
   	  motor_fix_in();
      Process_Data_From_Master_uP();  /* You edit this in user_routines.c */

      Update_OI();  // Update the OI LEDs and dashboard display
      COMPRESSOR = !rc_dig_in14;  /* Power pump only if pressure switch is off. */


      if (competition_mode)  // Robot is disabled
      {
        notmoving = FALSE;
        AutoFirstTime=TRUE; // Reset autonomous whenever the robot is disabled
        Gyro();  // Initialize gyro
      }
      else if (autonomous_mode)  // Robot is in autonomous
      {
        notmoving = FALSE;
        User_Autonomous_Code();        /* You edit this in user_routines_fast.c */
      }
      else  // Robot is in normal human driver mode
      {
        User_Driver();
        AutoFirstTime=TRUE; // Reset autonomous whenever the auto switch is disabled
      }
#ifdef _FRC_BOARD
//      Generate_Pwms(pwm13,pwm14,pwm15,pwm16);
#endif
    	motor_fix_out(); /* Convert pwms to int and the range 0 to 254 */
     }
//  if (LastClock < Clocksec)
//  {
//    LastClock = Clocksec;
//  }      
      Putdata(&txdata);             /* DO NOT CHANGE! */
    }

    if (warmUp == 0) // digital & OI inputs are suspect @ startup. Give them time to warm up
      Process_Data_From_Local_IO();     /* You edit this in user_routines_fast.c */
                                      /* I'm fast!  I execute during every loop.*/
  } /* while (1) */
}  /* END of Main */



/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
