/*******************************************************************************
* FILE NAME: motor_routines.c
*
* DESCRIPTION:
*  This file contains motor/joystick related filters.
*
* USAGE:
*
*******************************************************************************/

#ifdef _DEBUG
#include "print_routines.h"
#endif

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "timers.h"

#include "user_routines.h"
#include "clock_routines.h"
#include "motor_routines.h"
#include "auto_routines.h"
#include "sensor_routines.h"

/*******************************************************************************
* FUNCTION NAME: abs
* PURPOSE:       Return absolute value
*******************************************************************************/
unsigned int abs(signed int inb) 
{
  if(inb >= 0)
      return inb;
  else
      return -inb;
}


/*******************************************************************************
* FUNCTION NAME: motor_fix_in
* PURPOSE:       Converts pwm 0 - 254 range to -127 to 127
 *******************************************************************************/
void motor_fix_in(void)
{
	ip1_y = (signed int) p1_y - 127;
	ip2_y = (signed int) p2_y - 127;
	ip3_y = (signed int) p3_y - 127;
	ip4_y = (signed int) p4_y - 127;
	ip1_x = (signed int) p1_x - 127;
	ip2_x = (signed int) p2_x - 127;
	ip3_x = (signed int) p3_x - 127;
	ip4_x = (signed int) p4_x - 127;
    ip1_wheel = (signed int) p1_wheel - 127;
    ip2_wheel = (signed int) p2_wheel - 127;   
    ip3_wheel = (signed int) p3_wheel - 127;   
    ip4_wheel = (signed int) p4_wheel - 127; 
    ip1_aux = (signed int) p1_aux - 127;
    ip2_aux = (signed int) p2_aux - 127;   
    ip3_aux = (signed int) p3_aux - 127;   
    ip4_aux = (signed int) p4_aux - 127; 
}

/*******************************************************************************
* FUNCTION NAME: motor_fix_out
* PURPOSE:       Converts pwm -127 to 127 range to 0 - 254
*******************************************************************************/
void motor_fix_out(void)
{
  if (ipwm01 < -127) ipwm01=-127;
  if (ipwm02 < -127) ipwm02=-127;
  if (ipwm03 < -127) ipwm03=-127;
  if (ipwm04 < -127) ipwm04=-127;
  if (ipwm05 < -127) ipwm05=-127;
  if (ipwm06 < -127) ipwm06=-127;
  if (ipwm07 < -127) ipwm07=-127;
  if (ipwm08 < -127) ipwm08=-127;
  if (ipwm09 < -127) ipwm09=-127;
  if (ipwm10 < -127) ipwm10=-127;
  if (ipwm11 < -127) ipwm11=-127;
  if (ipwm12 < -127) ipwm12=-127;
  if (ipwm13 < -127) ipwm13=-127;
  if (ipwm14 < -127) ipwm14=-127;
  if (ipwm15 < -127) ipwm15=-127;
  if (ipwm16 < -127) ipwm16=-127;

  if (ipwm01 > 127) ipwm01=127;
  if (ipwm02 > 127) ipwm02=127;
  if (ipwm03 > 127) ipwm03=127;
  if (ipwm04 > 127) ipwm04=127;
  if (ipwm05 > 127) ipwm05=127;
  if (ipwm06 > 127) ipwm06=127;
  if (ipwm07 > 127) ipwm07=127;
  if (ipwm08 > 127) ipwm08=127;
  if (ipwm09 > 127) ipwm09=127;
  if (ipwm10 > 127) ipwm10=127;
  if (ipwm11 > 127) ipwm11=127;
  if (ipwm12 > 127) ipwm12=127;
  if (ipwm13 > 127) ipwm13=127;
  if (ipwm14 > 127) ipwm14=127;
  if (ipwm15 > 127) ipwm15=127;
  if (ipwm16 > 127) ipwm16=127;

	pwm01 = (unsigned char) ipwm01 +127;
	pwm02 = (unsigned char) ipwm02 +127;
	pwm03 = (unsigned char) ipwm03 +127;
	pwm04 = (unsigned char) ipwm04 +127;
	pwm05 = (unsigned char) ipwm05 +127;
	pwm06 = (unsigned char) ipwm06 +127;
	pwm07 = (unsigned char) ipwm07 +127;
	pwm08 = (unsigned char) ipwm08 +127;
	pwm09 = (unsigned char) ipwm09 +127;
	pwm10 = (unsigned char) ipwm10 +127;
	pwm11 = (unsigned char) ipwm11 +127;
	pwm12 = (unsigned char) ipwm12 +127;
	pwm13 = (unsigned char) ipwm13 +127;
	pwm14 = (unsigned char) ipwm14 +127;
	pwm15 = (unsigned char) ipwm15 +127;
	pwm16 = (unsigned char) ipwm16 +127;
}



/*******************************************************************************
* FUNCTION NAME: Limit_Mix
* PURPOSE:       Limits the mixed value for one joystick drive.
* CALLED FROM:   Default_Routine, this file
* ARGUMENTS:     
*     Argument             Type    IO   Description
*     --------             ----    --   -----------
*     intermediate_value    int    I    
* RETURNS:       unsigned char
*******************************************************************************/
int Limit_Mix (int intermediate_value)
{
  static int limited_value;
  
  if (intermediate_value < -127)
  {
    limited_value = -127;
  }
  else if (intermediate_value > 127)
  {
    limited_value = 127;
  }
  else
  {
    limited_value = intermediate_value;
  }
  return (int) (limited_value);
}



/*******************************************************************************
* FUNCTION NAME: Continuous_Deadband
* PURPOSE:       This filter establishes a bigger deadspot for the joysticks.
*                i.e., instead of neutral being exactly 0, it is -10 to 10.
*                so the motor is sent values of -127 to -10 or 10 to 127.
*                It also accounts for the Victor deadspot.
* CALLED FROM:   User_Drive() 
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Continuous_Deadband(int *p1)
{
#define DeadZone  20    // Joystick deadzone is at zero +/- this value
#define MAXPOWER 105   // Apply max power past this point

  if ((*p1 > -DeadZone) && (*p1 < DeadZone))
  {
    *p1 = 0;  // Neutral
  }
  else if (*p1 >= DeadZone)
  {
    *p1 -= DeadZone;
  }
  else
  {
    *p1 += DeadZone;
  }

  if (*p1 > MAXPOWER)
  {
    *p1 = 127;  // Force to max power
  }
  else if (*p1 < -MAXPOWER)
  { 
    *p1 = -127;  // Force to max power
  }
}


/*******************************************************************************
* FUNCTION NAME: Arm_Deadband
* PURPOSE:       This filter establishes a bigger deadspot for the joysticks.
*                i.e., instead of neutral being exactly 0, it is -10 to 10.
*                so the motor is sent values of -127 to -10 or 10 to 127.
*                It also accounts for the Victor deadspot.
* CALLED FROM:   User_Drive() 
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Arm_Deadband(int *p1)
{
#define ARMDEADZONE  10    // Joystick deadzone is at zero +/- this value
#define ARMMAXPOWER 100   // Apply max power past this point

  int temp;

  if ((*p1 > -ARMDEADZONE) && (*p1 < ARMDEADZONE))
  {
//    *p1 = 0;  // Neutral
    temp = 0;  // Neutral
  }
  else if (*p1 >= ARMDEADZONE)
  {
//    *p1 -= ARMDEADZONE;
//    *p1 = *p1 / 2;  // Reduce the arm response
    temp -= ARMDEADZONE;
    temp = *p1 / 2;  // Reduce the arm response
  }
  else
  {
//    *p1 += ARMDEADZONE;
//    *p1 = *p1 / 2;  // Reduce the arm response
    temp += ARMDEADZONE;
    temp = *p1 / 2;  // Reduce the arm response
  }

  if (*p1 > ARMMAXPOWER)
  {
//    *p1 = 127;  // Force to max power
    temp = 127;  // Force to max power
  }
  else if (*p1 < -ARMMAXPOWER)
  { 
//    *p1 = -127;  // Force to max power
    temp = -127;  // Force to max power
  }
  *p1 = temp;
}



/*******************************************************************************
* FUNCTION NAME: Smooth_Joystick
* PURPOSE:       This filter changes the joystick respinse curve and establishes
*                a bigger deadspot for the joysticks.
*                i.e., instead of neutral being exactly 0, it is -10 to 10.
*                so the motor is sent values of -127 to -10 or 10 to 127.
*                It also avoids the Victor deadspot.
* NOTES:         The Victor 884 range is: 0-37   = Full reverse
*                                        123-136 = Neutral
*                                        227-254 = Full Forward
*                In our terms:  0-9(4)   = Neutral
*                             (90)100-127 = Full  (!!I've noticed 110+ isn't full!!)
* CALLED FROM:   User_Drive() 
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
const rom signed char Joystick_Curve[128] = 
{
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   1,   1,   1,   1,   2, 
  2,   2,   3,   3,   3,   4,   4,   4,   5,   5,   6,   6,   7,   7,   8,   8, 
  9,   9,  10,  10,  11,  12,  12,  13,  14,  14,  15,  16,  16,  17,  18,  19, 
 19,  20,  21,  22,  23,  24,  25,  25,  26,  27,  28,  29,  30,  31,  32,  33, 
 34,  35,  36,  37,  38,  39,  40,  42,  43,  44,  45,  46,  47,  49,  50,  51, 
 52,  54,  55,  56,  57,  59,  60,  61,  63,  64,  66,  67,  68,  70,  71,  73, 
 74,  76,  77,  79,  80,  82,  83,  85,  86,  88,  90,  91,  93,  94,  96,  98, 
100, 101, 103, 105, 106, 108, 110, 112, 114, 115, 117, 119, 121, 123, 125, 127
};

void Smooth_Joystick(int *p1)
{
//  int temp;

  if(*p1 < 0)
     *p1 = -1 * Joystick_Curve[abs(*p1)];
  else
     *p1 = Joystick_Curve[*p1];

//  temp = abs(*p1);
//  temp = Joystick_Curve[temp];
//  if(*p1 < 0)
//    temp = -temp;
//  *p1 = temp;
}


/*******************************************************************************
* FUNCTION NAME: Hold_Position
* PURPOSE:       Apply motor power to hold position if necessary.
*                This is also used to brake in User Driver mode, but will have
*                difficulty reaching a steady-state without a turn movement, 
*                because the minimum power for a turn is so high, e.g., PWM 70 vs.
*                PWM 20 and you'd need a counter-intuitive movement AWAY from the setpoint.
*                (When pushed by a person it reaches a max of ~70 pwm to counter)
* CALLED FROM:   User_Drive() & Brake()
*******************************************************************************/
#define KP 7/10  // Basic proportional adjustment (7/10 works)
#define KD 8  // Basic derivative adjustment (8 works)
#define KI 10  // Basic integral adjustment (0 works)
/*  KP = 8/10, KD=9 is too unstable */
#define T  40 // This is executed at the slow loop rate of 40 Hz
#define ReverseStep 2  // Maximum allowed power increase during the first cycles
                       // Allows for gradual rather than abrupt braking
#define CUMLIMIT 20000  // Limit potential integral wind-up
#define INTEGRAL_WINDOW 10  //Integral window +/- to stop correcting

void Hold_Position(char init, int HoldLeft, int HoldRight)
{
  int pL, pR, dL, dR, iR, iL;
  static int LastActualL, LastActualR, ReverseLimit=127;
  static int CumErrorR, CumErrorL, Rerror, Lerror;

  if (init)
  { // Hold robot at these encoder values
    LastActualL = 0;
    LastActualR = 0;
    CumErrorR = 0;
    CumErrorL = 0;
    ReverseLimit = 20; // Use 127 if no ramp-up is desired
    PrintString((char *) "Hold_Position");
    PrintCR();
  }
  else
  {
    if (PadsCounter < 20) // Don't attempt to hold if bouncing down off the pads
    {
      ++PadsCounter;
    }
    else
    {
    PadsCounter = 21; //Avoid overflow
    Rerror = (HoldRight - Right_Encoder_Count);
    Lerror = (HoldLeft  - Left_Encoder_Count);
    /* Proportional */
    /* Add correction proportional to the current error */
    /* Might want to match Left side response to Right */
    pR = Rerror * (int)KP;
    pL = Lerror  * (int)KP;

    /* Derivative */
    /* Sudden reaction to sudden changes */
    dR = (Right_Encoder_Count - LastActualR) * (int)KD;
    dL = (Left_Encoder_Count - LastActualL) * (int)KD;
    LastActualL = Left_Encoder_Count;
    LastActualR = Right_Encoder_Count;

    /* Integral */
    /* Correct accumulated error */
//    iR = CumErrorR * (int)KI / (int)T; // p covers current error
//    iL = CumErrorL * (int)KI / (int)T; // p covers current error

    /* Check for runaway accumulated error */
//    if ((Rerror < INTEGRAL_WINDOW) && (Rerror > -INTEGRAL_WINDOW))
//      if ((Lerror < INTEGRAL_WINDOW) && (Lerror > -INTEGRAL_WINDOW))
//      {
//        CumErrorR += Rerror; 
//        if (CumErrorR > (int)CUMLIMIT) // Detect and avoid integral windup
//          CumErrorR = (int)CUMLIMIT;
//        if (CumErrorR < (int)-CUMLIMIT)// Detect and avoid integral windup
//          CumErrorR = (int)-CUMLIMIT;
//        CumErrorL += Lerror;
//        if (CumErrorL > (int)CUMLIMIT) // Detect and avoid integral windup
//          CumErrorL = (int)CUMLIMIT;
//        if (CumErrorL < (int)-CUMLIMIT)// Detect and avoid integral windup
//          CumErrorL = (int)-CUMLIMIT;
//      }

//    WHL_R1 = pR - dR + iR; // could apply i to force robot completely back into position
//    WHL_L1 = pL - dL + iL;
    WHL_R1 = pR - dR; // could apply i to force robot completely back into position
    WHL_L1 = pL - dL;

    /* Keep the motors within permitted range */
  if ((WHL_R1 > -NULLJOYSTICK) && (WHL_R1 < NULLJOYSTICK))
    WHL_R1 = 0;  // Neutral

  if ((WHL_L1 > -NULLJOYSTICK) && (WHL_L1 < NULLJOYSTICK))
    WHL_L1 = 0;  // Neutral

    if (WHL_R1 < -127) WHL_R1 = -127;
    if (WHL_L1 < -127) WHL_L1 = -127;
    if (WHL_R1 > 127) WHL_R1 = 127;
    if (WHL_L1 > 127) WHL_L1 = 127;

    /* Apply ramp-up to full power rather than jumping right to full power */
    ReverseLimit += ReverseStep;
    if (ReverseLimit > 127) ReverseLimit = 127;
    if (WHL_R1 < -ReverseLimit) WHL_R1 = -ReverseLimit;
    if (WHL_L1 < -ReverseLimit) WHL_L1 = -ReverseLimit;
    if (WHL_R1 > ReverseLimit) WHL_R1 = ReverseLimit;
    if (WHL_L1 > ReverseLimit) WHL_L1 = ReverseLimit;

    WHL_R2 = WHL_R1;
    WHL_L2 = WHL_L1;
#ifdef _DEBUG
//PrintCR();
//PrintString((char *) ", HOLD:");
//PrintString((char *) ", Right="); PrintSignedDecimal(Right_Encoder_Count);
//PrintString((char *) ", Left="); PrintSignedDecimal(Left_Encoder_Count);
//PrintString((char *) ", pR="); PrintSignedDecimal(pR);
//PrintString((char *) ", pL="); PrintSignedDecimal(pL);
//PrintString((char *) ", iR="); PrintSignedDecimal(iR);
//PrintString((char *) ", iL="); PrintSignedDecimal(iL);
//PrintString((char *) ", dR="); PrintSignedDecimal(dR);
//PrintString((char *) ", dL="); PrintSignedDecimal(dL);
//PrintString((char *) ", WHL="); PrintSignedDecimal(WHL_L1);
//PrintString((char *) ", WHR="); PrintSignedDecimal(WHL_R1);
//PrintCR();
#endif
    }
  }
}



/*******************************************************************************
* FUNCTION NAME: Brake
* PURPOSE:       Apply motor power to slow down.
* CALLED FROM:   User_Drive() 
*******************************************************************************/
#define BKP 2/10  // Basic proportional adjustment pwr/ticks, e.g., pwm 2 @ 30 ticks from setpoint (2/10 works)
#define BKD 0  // Basic derivative adjustment doesn't apply in this situation (0 works)
#define BKI 20  // Basic integral adjustment (20 works)
#define BT  40 // This is executed at the slow loop rate of 40 Hz
#define BCUMLIMIT 2000  // Limit to error accumulation
#define BReverseStep 1  // Maximum allowed power increase during the first cycles
                       // Allows for gradual rather than abrupt braking
void Brake(char init, int HoldLeft, int HoldRight)
{
  long pL, pR, dL, dR, iR, iL, Lerror, Rerror;
  static long LastActualL, LastActualR, CumErrorR, CumErrorL, ReverseLimit;
  static char LPositive, RPositive, Positive, HoldFlag, HoldCount;

  if (init)
  { // Hold robot at these encoder values
    LastActualL = 0;
    LastActualR = 0;
    ReverseLimit = 10;
    CumErrorL = 0;
    CumErrorR = 0;
    HoldCount = 0;
    HoldFlag = FALSE;
    if (HoldRight > Right_Encoder_Count)
      RPositive = TRUE;
    else
      RPositive = FALSE;
    if (HoldLeft > Left_Encoder_Count)
      LPositive = TRUE;
    else
      LPositive = FALSE;
    Positive = RPositive;
    Hold_Position((int)TRUE, (int)HoldRight, (int)HoldRight);

  }
  else
  {
   if (Positive & (HoldRight < Right_Encoder_Count)) // Switch to Hold after passing setpoint
   {
      HoldFlag = TRUE;
      Positive = FALSE;
   }
   else if (!Positive & (HoldRight > Right_Encoder_Count))
   {
      HoldFlag = TRUE;
      Positive = TRUE;
   }

   if (HoldFlag)
   {
      Hold_Position((int)FALSE, (int)HoldRight, (int)HoldRight);
      HoldCount++;
      LastActualR = Right_Encoder_Count; // Maintain for possible later use
      LastActualL = Left_Encoder_Count;  // Maintain for possible later use
      if (HoldCount > 25) // Only Hold Position for a limited time to get near the stop point
      {
        HoldFlag = FALSE;
        HoldCount = 0;
        CumErrorL = 0;
        CumErrorR = 0;
      }
   }
   else
   {
/**********************************************************************/
    /* Proportional */
    /* Add correction proportional to the current error */
    /* Might want to match Left side response to Right */
    Rerror = (HoldRight - Right_Encoder_Count);
    Lerror = (HoldLeft - Left_Encoder_Count);
    pR = Rerror * (long)BKP;
    pL = Lerror * (long)BKP;
PrintCR();
PrintString((char *) ", Rerror1=");
PrintSignedDecimal(Rerror);
/**********************************************************************/
    /* Integral */
    /* Correct accumulated error */
    /* Only accumulate if position isn't changing AND isn't correct */

    if (Rerror == 0)
      CumErrorR = 0;
    if (Lerror == 0)
      CumErrorL = 0;
      
    if (RPositive & (Rerror < 0))  // If we pass target then disgard accumulated error
    {
       RPositive = FALSE;
       CumErrorR = 0;
    }
    if (!RPositive & (Rerror > 0))  // If we pass target then disgard accumulated error
    {
       RPositive = TRUE;
       CumErrorR = 0;
    }
    if (LPositive & (Lerror < 0))  // If we pass target then disgard accumulated error
    {
       LPositive = FALSE;
       CumErrorL = 0;
    }
    if (!LPositive & (Lerror > 0))  // If we pass target then disgard accumulated error
    {
       LPositive = TRUE;
       CumErrorL = 0;
    }
PrintString((char *) ", Rerror2=");
PrintSignedDecimal(Rerror);

    iR = CumErrorR * (long)BKI / (long)BT; // p covers current error
    iL = CumErrorL * (long)BKI / (long)BT; // p covers current error

    if (LastActualR == Right_Encoder_Count) // No movement detected
      CumErrorR += Rerror;

    if (LastActualL == Left_Encoder_Count) 
      CumErrorL += Lerror;

PrintString((char *) ", LastActualR=");
PrintSignedDecimal(LastActualR);
PrintString((char *) ", Right_Encoder_Count=");
PrintSignedDecimal(Right_Encoder_Count);
PrintCR();

      LastActualR = Right_Encoder_Count; // Maintain for possible later use
      LastActualL = Left_Encoder_Count;  // Maintain for possible later use

    /* Check for runaway accumulated error */
    if (CumErrorR > (long)BCUMLIMIT) // Detect and avoid integral windup
      CumErrorR = (long)BCUMLIMIT;
    if (CumErrorR < (long)-BCUMLIMIT)// Detect and avoid integral windup
      CumErrorR = (long)-BCUMLIMIT;

    if (CumErrorL > (long)BCUMLIMIT) // Detect and avoid integral windup
      CumErrorL = (long)BCUMLIMIT;
    if (CumErrorL < (long)-BCUMLIMIT)// Detect and avoid integral windup
      CumErrorL = (long)-BCUMLIMIT;

/**********************************************************************/
    /* Derivative */
    /* Sudden reaction to sudden changes */
    /* Doesn't really apply to the braking situation */
//    dR = (Right_Encoder_Count - LastActualR) * (long)BKD;
//    dL = (Left_Encoder_Count - LastActualL) * (long)BKD;
//    LastActualR = Right_Encoder_Count;
//    LastActualL = Left_Encoder_Count;
/**********************************************************************/

    WHL_R1 = pR + iR;
    WHL_L1 = pL + iL;
//    WHL_R1 = pR;
//    WHL_L1 = pL;

    /* Ease the transition into full power reverse */
    ReverseLimit += BReverseStep;
    if (ReverseLimit > 127) ReverseLimit = 127;
    if (WHL_R1 < -ReverseLimit) WHL_R1 = -ReverseLimit;
    if (WHL_L1 < -ReverseLimit) WHL_L1 = -ReverseLimit;
    if (WHL_R1 > ReverseLimit) WHL_R1 = ReverseLimit;
    if (WHL_L1 > ReverseLimit) WHL_L1 = ReverseLimit;

#ifdef _DEBUG
PrintCR();
PrintString((char *) "BRAKE:");
PrintString((char *) ", pR="); PrintSignedDecimal(pR);
//PrintString((char *) ", dR="); PrintSignedDecimal(dR);
PrintString((char *) ", iR="); PrintSignedDecimal(iR);
PrintString((char *) ", Rcum="); PrintSignedDecimal(CumErrorR);
PrintString((char *) ", Rencd="); PrintSignedDecimal(Right_Encoder_Count);
PrintString((char *) ", WR="); PrintSignedDecimal(WHL_R1);
PrintCR();

PrintString((char *) ", pL="); PrintSignedDecimal(pL);
//PrintString((char *) ", dL="); PrintSignedDecimal(dL);
PrintString((char *) ", iL="); PrintSignedDecimal(iL);
PrintString((char *) ", Lcum="); PrintSignedDecimal(CumErrorL);
PrintString((char *) ", Lencd="); PrintSignedDecimal(Left_Encoder_Count);
PrintString((char *) ", WL="); PrintSignedDecimal(WHL_L1);
PrintCR();
#endif
   }
  }
}


/*******************************************************************************
* FUNCTION NAME: PWMBalance
* PURPOSE:       Return the values needed to balance the two sides
*                of the drivetrain while driving.
*                Normally PID would correct for this, but this routine benefits us
*                by starting off closer to the correct values and in the event of
*                sensor failures will keep the robot close to course.
*******************************************************************************/
  /* The L/R motor balancing PWM adjustments are for PWMs: 
                                            0, 20, 30, 40, 50, 60, 70, 80, 90, 100, 110, 120, 127 */
/*-------------------------------------------------------------------------------------------*/
rom	const int PwmBalanceForwardLeft[13] =  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
rom	const int PwmBalanceForwardRight[13] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
rom const int PwmBalanceBackwardLeft[13] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
rom const int PwmBalanceBackwardRight[13]= {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
int PWMBalance(char direction, int speed)
{
  int tempspeed;
  
  /* Generate table lookup index */
  tempspeed = speed + 5;  // Round up
  tempspeed = (tempspeed / 10) - 1; // Convert to index
  if (tempspeed < 0) tempspeed = 0;  // Minimum index value
  if (tempspeed > 12) tempspeed = 12;  // Maximum index value

  if (direction == BalForLeft)
  {
    tempspeed = PwmBalanceForwardLeft[tempspeed];
  }
  else if (direction == BalForRight)
  {
    tempspeed = PwmBalanceForwardRight[tempspeed];
  }
  else if (direction == BalBckLeft)
  {
    tempspeed = PwmBalanceBackwardLeft[tempspeed];
  }
  else if (direction == BalBckRight)
  {
    tempspeed = PwmBalanceBackwardRight[tempspeed];
  }

  return (tempspeed);
}



/*******************************************************************************
* FUNCTION NAME: ArmTelescope
* PURPOSE:       Control and protect the arm
*                Analog 3 is the potentiometer
*                PWM 6 - Fisher Price
*******************************************************************************/

int ArmTelescope(char direction, int speed)
{
  int arm_wench_pot;

  arm_wench_pot = Get_Analog_Value(rc_ana_in03);

  if (arm_wench_pot > 50)
  {
  }
  else if (arm_wench_pot < 600)
  {
  }
}


/*******************************************************************************
* FUNCTION NAME: ArmJoint
* PURPOSE:       Control and protect the arm
*                Analog 2 is the potentiometer
*                PWM 5 - Van door
*******************************************************************************/

int ArmJoint(char direction, int speed)
{
}



/*******************************************************************************
* FUNCTION NAME: Hold_Arm_Position
* PURPOSE:       Hold the arm in a stationary position
*                Assume ARM_INPUT is neutral, but that neutral may not be zero
* INPUTS/OUTPUTS init - initialized hold position (pot setting)
*                setting - the pot position to maintain
*                arm_output - pwm (0 neutral) to the arm pivot motor
*******************************************************************************/
#define ARM_KP 10  // Basic proportional adjustment in tenths (? works)
#define ARM_KD 0  // Basic derivative adjustment (? works)
#define ARM_KI 0  // Basic integral adjustment (? works)
#define ARM_T  40 // This is executed at the slow loop rate of 40 Hz

void Hold_Arm_Position(char init, int setting, int *arm_output)
{
  int pA, dA, iA;
  int Kp=ARM_KP, Ki=ARM_KD, Kd=ARM_KI;
  int arm_pot;
  static int LastActual, desired_pot;
  static int CumError, error;
  static int counter=0;

#ifdef MLM
  if (OISWITCH3) // Override hardcoded settings to tune from OI
  {
    Kp=OI_KP_POT>>1; //smooth out the range 0 to 12.7
    Ki=OI_KI_POT>>1; // 0 - 127
    Kd=OI_KD_POT>>1; // 0 - 127

    PrintString((char *) " OI SWITCH 3 is ON **** OI pot OVERRIDE **** ");
    PrintString((char *) "Kp= ");
    PrintSignedDecimal(Kp);
    PrintString((char *) ", Ki= ");
    PrintSignedDecimal(Ki);
    PrintString((char *) ", Kd= ");
    PrintSignedDecimal(Kd);
    PrintCR();
  }
#endif

  if (init)
  { // Hold robot at these encoder values
    desired_pot = setting;
    LastActual = 0;
    CumError = 0;
    // arm_output remains unchanged at neutral
    PrintString((char *) "Hold_Arm_Position");
    PrintCR();
  }
  else
  {
    arm_pot = Get_Analog_Value(ARM_POT);
    error = (desired_pot - arm_pot);

    /* Proportional */
    /* Add correction proportional to the current error */
    /* Might want to match Left side response to Right */
    pA = error * (int)Kp/10;

    /* Derivative */
    /* Sudden reaction to sudden changes */
    dA = (arm_pot - LastActual) * (int)Kd;
    LastActual = arm_pot;

    /* Integral */
    /* Correct accumulated error */
    iA = CumError * (int)Ki / (int)ARM_T; // p covers current error

    *arm_output = ARMNEUTRAL + pA - dA + iA; // Add arm neutral because it probably won't be zero
/*** To reverse arm direction need to reverse the arm_output? ***/
//    *arm_output *= -1;

counter++;
if (counter == 40)
{
//  PrintString((char *) "arm_output=");
//  PrintSignedDecimal(arm_output);
//  PrintCR();
}
  }
}    
