/*******************************************************************************
*
*	TITLE:		odometry_routines.c
*	VERSION:	1.1
*	DATE:		19-Feb-2005
*	AUTHOR:		Robert Frank
*				admin@bobbys.us
*
*******************************************************************************/

#include "sensor_routines.h"
#include "odometry_routines.h"
#include "user_routines.h"
#include "sine.h"
#include "ifi_utilities.h"

#ifdef _DEBUG
#include "print_routines.h"
#endif

/*******************************************************************************
* FUNCTION NAME: Recompute_XY
* PURPOSE:       this is called after a certain time interval to recompute 
*                 the location of the robot from the origional position
*                 using the encoders
* CALLED FROM:   wherever
* ARGUMENTS:     increments of the left and right encoder since last call.
* RETURNS:       void
*******************************************************************************/

void Recompute_XY(signed long Left_Encoder_Total, signed long Right_Encoder_Total) //int incl, int incr)
{
#ifdef _DEBUG
    static int bcount=0;
#endif
	//incl/r is the number of increments since last called
    static signed long prev_encoder_left=0, prev_encoder_right=0;
    static long x_pos=0,y_pos=0;
    long angle;
    int incl=0, incr=0;
    long U_r=0,U_l=0;	//change in distance for left and right * TICKS_PER_REV
    incl = (signed int)(Left_Encoder_Total - prev_encoder_left);
    incr = (signed int)(Right_Encoder_Total - prev_encoder_right);
    U_r = (ENCODER_INCHES * incr);//conversion factor=ENCODER_INCHES/TICKS_PER_REV
    U_l = (ENCODER_INCHES * incl); //max = 72416175
    //m_U = (U_r + U_l) / 2;
    angle = (long)(GYRO_DEGREES/10);//(U_r - U_l) / 20480; //(24*25600)=614400//(614400/30) = 20480
    x_pos += (long)( ( (U_r + U_l) * cos(angle)) /650240);//tenths of an inch!!!! (( ((U_r + U_l) / 256) * cos(angle)) / 25400); //(2*127*100) = 25400
    y_pos += (long)( ( (U_r + U_l) * sin(angle)) /650240);//tenths of an inch!!!! (( ((U_r + U_l) / 256) * sin(angle)) / 25400); //(2*25600)=51200 //max = 4294967295
#ifdef _DEBUG
    bcount++;
    if(bcount==20)
    {
    PrintString((char *)"l = ");
    PrintSignedDecimal(incl);
    PrintCR();
    PrintString((char *)"r  = ");
    PrintSignedDecimal(incr);
    PrintCR();
    PrintString((char *)"X_POS = ");
    PrintSignedDecimal(x_pos); //tenths of an inch
    PrintCR();
    PrintString((char *)"Y_POS = ");
    PrintSignedDecimal(y_pos);
    PrintCR();
    PrintString((char *)"Angle = "); //tenths of a degree
    PrintSignedDecimal(GYRO_DEGREES);
    PrintCR();
    PrintString((char *)"Sine(Angle)");
    PrintSignedDecimal(sin(angle));
    PrintCR();
    PrintString((char *)"Cosine(Angle) = ");
    PrintSignedDecimal(cos(angle));
    PrintCR();
    bcount=0;
    }
#endif
    prev_encoder_left = Left_Encoder_Total;
    prev_encoder_right = Right_Encoder_Total;
}
