/*******************************************************************************
* FILE NAME: user_routines.h
*
* DESCRIPTION: 
*  This is the include file which corresponds to user_routines.c and
*  user_routines_fast.c
*  It contains some aliases and function prototypes used in those files.
*
* USAGE:
*  If you add your own routines to those files, this is a good place to add
*  your custom macros (aliases), type definitions, and function prototypes.
*******************************************************************************/

#ifndef __user_routines_h_
#define __user_routines_h_


/*******************************************************************************
                            MACRO DECLARATIONS
*******************************************************************************/

/* Used in limit switch routines in user_routines.c */
#define OPEN        1     /* Limit switch is open (input is floating high). */
#define CLOSED      0     /* Limit switch is closed (input connected to ground). */

#define FALSE             0
#define TRUE              1
#define ON                1
#define OFF               0

#define WHL_L1	ipwm01
#define WHL_L2	ipwm02
#define WHL_R1	ipwm03
#define WHL_R2	ipwm04
#define JOY_DRIVE1A	ip2_y // Both 1 & 2 joystick driving
#define JOY_DRIVE1B	ip2_x // 1 joystick driving
#define JOY_DRIVE2	ip1_y // 2 joystick driving

#define ARM_PWM	ipwm05
//#define ARM_PWM	ipwm03  //MLM
#define ARM_INPUT	ip3_y
//#define ARM_POT  rc_ana_in16  //MLM
#define ARM_POT  rc_ana_in02
//#define ARMNEUTRAL -20
#define ARMNEUTRAL 0     //MLM

#define TELESCOPE_PWM1	ipwm06
#define TELESCOPE_POT  rc_ana_in03

#define FIELD_LEFT   rc_dig_in11
#define FIELD_CENTER rc_dig_in12
#define FIELD_RIGHT  rc_dig_in13

/************ OI Inputs ************/
#define OISWITCH1 p4_sw_aux1  // Used to choose 1 or 2 joystick drive
#define OISWITCH2 p4_sw_aux2  // Disable Arm limits
#define OISWITCH3 p4_sw_trig  // unassigned
#define OISWITCH4 p4_sw_top   // unassigned
#define OISWITCH5 p4_sw_aux1  // unassigned
#define OISWITCH6 p4_sw_aux2  // unassigned
#define OISWITCH7 p1_sw_trig  // unassigned
#define OISWITCH8 p1_sw_top   // unassigned
#define OISWITCH9 p1_sw_aux1  // unassigned
#define OISWITCH10 p1_sw_aux2 // unassigned

#define OI_KP_POT p2_x  // PID tuning
#define OI_KI_POT p2_y  // PID tuning
#define OI_KD_POT p2_wheel  // PID tuning
#define OI_POT4 p2_aux  // unassigned

/* On DRIVE joystick */
#define PUSH_DOWN_PADS        relay6_fwd
#define PUSH_DOWN_PADS_REV    relay6_rev
#define PUSH_DOWN_PADS_INPUT  p2_sw_trig

/* On ARM joystick */
#define TETRA_GRABBER      relay7_fwd
#define TETRA_GRABBER_REV  relay7_rev
#define COMPRESSOR         relay8_fwd

#define TETRA_GRABBER_INPUT      p3_sw_trig
//#define TELESCOPE_IN_INPUT	     p3_sw_aux1
//#define TELESCOPE_OUT_INPUT	     p3_sw_aux2
#define TELESCOPE_IN_INPUT	     OISWITCH4
#define TELESCOPE_OUT_INPUT	     OISWITCH3

/*******************************************************************************
                            TYPEDEF DECLARATIONS
*******************************************************************************/


/*******************************************************************************
                        GLOBAL VARIABLE DECLARATIONS
*******************************************************************************/
#ifdef VARIABLE_DECLARE
      /** One encoder revolution (256 interrupts) = 11.05" of travel **/
      /*  One encoder "tick" = .043" */
  signed long Right_Encoder_Count=0,Left_Encoder_Count=0;
  signed long Total_Right_Encoder_Count=0,Total_Left_Encoder_Count=0;
  char Beacon_type=0;
  char AutoGoal=0; // FALSE by default, goal waits for first user trigger
  char AutoFirstTime;  // Identifies when Autonomous mode first starts
  char notmoving=FALSE;
  unsigned char PadsCounter=0;
#else
  extern signed long Right_Encoder_Count,Left_Encoder_Count;
  extern signed long Total_Right_Encoder_Count,Total_Left_Encoder_Count;
  extern char Beacon_type;
  extern char AutoGoal;
  extern char AutoFirstTime;
  extern char notmoving;
  extern unsigned char PadsCounter;
#endif


/*******************************************************************************
                           FUNCTION PROTOTYPES
*******************************************************************************/

/* These routines reside in user_routines.c */
void User_Initialization(void);
void Process_Data_From_Master_uP(void);
void User_Driver(void);
void Update_OI(void);

/* These routines reside in user_routines_fast.c */
void InterruptHandlerLow (void);  /* DO NOT CHANGE! */
void User_Autonomous_Code(void);  /* Only in full-size FRC system. */
void Process_Data_From_Local_IO(void);

#endif
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
